//
//  DiaryDocument.m
//  Vermont Recipes 2.0.0
//
//  Created by Bill Cheeseman on 2009-06-03.
//  Copyright 2000-2009 Bill Cheeseman. All rights reserved.
//
//  CREATED IN RECIPE 3

#import "DiaryDocument.h"
#import "DiaryWindowController.h" // ADDED IN RECIPE 3
#import "VRDocumentController.h" // ADDED IN RECIPE 6

#pragma mark MACROS

#define DIARY_TITLE_MARKER	@"\u270E" // Unicode LOWER RIGHT PENCIL // ADDED IN RECIPE 4
#define DIARY_TAG_MARKER	@"\u2690" // Unicode WHITE FLAG // ADDED IN RECIPE 4


#pragma mark -

@implementation DiaryDocument

#pragma mark INITIALIZATION

- (id)initForURL:(NSURL *)absoluteDocumentURL withContentsOfURL:(NSURL *)absoluteDocumentContentsURL ofType:(NSString *)typeName error:(NSError **)outError { // ADDED IN RECIPE 7
	if ((self = [super initForURL:absoluteDocumentURL withContentsOfURL:absoluteDocumentContentsURL ofType:typeName error:outError])) {
		isRestoringAutosavedDocument = YES; // reset in -[DiaryWindowController windowDidLoad]
	}
	return self;
}

#pragma mark ACCESSOR METHODS

- (void)setDiaryDocTextStorage:(NSTextStorage *)textStorage { // ADDED IN RECIPE 3
	if (diaryDocTextStorage != textStorage) {
		[diaryDocTextStorage release];
		diaryDocTextStorage = [textStorage retain];
	}
}

- (NSTextStorage *)diaryDocTextStorage { // ADDED IN RECIPE 3
	return [[diaryDocTextStorage retain] autorelease];
}

- (void)setIsRestoringAutosavedDocument:(BOOL)flag { // ADDED IN RECIPE 7
	isRestoringAutosavedDocument = flag;
}

- (BOOL)isRestoringAutosavedDocument { // ADDED IN RECIPE 7
	return isRestoringAutosavedDocument;
}

- (void)setIsRevertingToSavedDocument:(BOOL)flag { // ADDED IN RECIPE 7
	isRevertingToSavedDocument = flag;
}

- (BOOL)isRevertingToSavedDocument { // ADDED IN RECIPE 7
	return isRevertingToSavedDocument;
}

#pragma mark ACTION METHODS

/* REMOVED IN RECIPE 8
- (IBAction)saveDocumentAsPDFTo:(id)sender { // ADDED IN RECIPE 8 (preliminary version)
	// Action method connected to the Save As PDF menu item. Calls -setNameFieldStringValue: new in Snow Leopard.
	NSSavePanel *savePanel = [NSSavePanel savePanel];
	NSWindow *window = [[[self windowControllers] objectAtIndex:0] window];
	if (floor(NSAppKitVersionNumber) > NSAppKitVersionNumber10_5) {
		[self printDocumentWithSettings:[NSDictionary dictionaryWithObjectsAndKeys:NSPrintSaveJob, NSPrintJobDisposition, nil] showPrintPanel:NO delegate:nil didPrintSelector:NULL contextInfo:NULL];
	} else {
		[savePanel setRequiredFileType:@"pdf"];
		[savePanel setCanSelectHiddenExtension:YES];
		[savePanel beginSheetForDirectory:nil file:defaultPDFFileName modalForWindow:window modalDelegate:self didEndSelector:@selector(savePanelDidEnd:returnCode:contextInfo:) contextInfo:NULL];
	}
}
*/

- (IBAction)saveDocumentAsPDFTo:(id)sender { // ADDED IN RECIPE 8 (final version)
	// Action method connected to the Save As PDF menu item.
	NSSavePanel *savePanel = [NSSavePanel savePanel];
	NSWindow *window = [[[self windowControllers] objectAtIndex:0] window];
	VRDocumentController *controller = [VRDocumentController sharedDocumentController];
	NSString *defaultPDFFileName = NSLocalizedString(@"Chef’s Diary", @"default name of Chef's Diary document");
#if MAC_OS_X_VERSION_MIN_REQUIRED >= MAC_OS_X_VERSION_10_6
		// Calls -setNameFieldStringValue: and -beginSheetModalForWindow:completionHandler:, new in Snow Leopard.
		if ([controller canOpenURL:[controller currentDiaryURL]]) {
			defaultPDFFileName = [[[controller currentDiaryURL] lastPathComponent] stringByDeletingPathExtension]; // may be nil
		}
		if (defaultPDFFileName) [savePanel setNameFieldStringValue:defaultPDFFileName];
		[savePanel setAllowedFileTypes:[NSArray arrayWithObject:@"pdf"]];
		[savePanel setCanSelectHiddenExtension:YES];
		[savePanel beginSheetModalForWindow:window completionHandler:^(NSInteger result) {
			if (result == NSFileHandlingPanelOKButton) {
				[self printDocumentWithSettings:[NSDictionary dictionaryWithObjectsAndKeys:NSPrintSaveJob, NSPrintJobDisposition, [savePanel URL], NSPrintJobSavingURL, nil] showPrintPanel:NO delegate:nil didPrintSelector:NULL contextInfo:NULL];
				if ([savePanel isExtensionHidden]) [[NSFileManager defaultManager] setAttributes:[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithBool:YES], NSFileExtensionHidden, nil] ofItemAtPath:[[savePanel URL] path] error:NULL];
			}
		}];
#else
	if ([controller canOpenURL:[controller currentDiaryURL]]) {
			defaultPDFFileName = [[[[controller currentDiaryURL] path] lastPathComponent] stringByDeletingPathExtension]; // may be nil
		}
		[savePanel setRequiredFileType:@"pdf"];
		[savePanel setCanSelectHiddenExtension:YES];
		[savePanel beginSheetForDirectory:nil file:defaultPDFFileName modalForWindow:window modalDelegate:self didEndSelector:@selector(savePanelDidEnd:returnCode:contextInfo:) contextInfo:NULL];
#endif
}

- (void)savePanelDidEnd:(NSSavePanel *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo { // ADDED IN RECIPE 8
	// didEndSelector called by -saveDocumentAsPDFTo: only under Leopard.
	if (returnCode == NSOKButton) {
		[self printDocumentWithSettings:[NSDictionary dictionaryWithObjectsAndKeys:NSPrintSaveJob, NSPrintJobDisposition, [sheet filename], NSPrintSavePath, nil] showPrintPanel:NO delegate:nil didPrintSelector:NULL contextInfo:NULL];
		if ([sheet isExtensionHidden]) [[NSFileManager defaultManager] setAttributes:[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithBool:YES], NSFileExtensionHidden, nil] ofItemAtPath:[sheet filename] error:NULL];
	}
}

#pragma mark OVERRIDE METHODS

- (void)makeWindowControllers { // ADDED IN RECIPE 3
	DiaryWindowController *controller = [[DiaryWindowController alloc] init];
	[self addWindowController:controller];
	[controller release];
}

/* DELETED IN RECIPE 3
- (NSString *)windowNibName {
    // Implement this to return a nib to load OR implement -makeWindowControllers to manually create your controllers.
    return @"DiaryDocument";
}
*/

- (NSData *)dataOfType:(NSString *)typeName error:(NSError **)outError { // ADDED IN RECIPE 3
    NSRange range = NSMakeRange(0, [[self diaryDocTextStorage] length]);
    NSData *data = [[self diaryDocTextStorage] RTFFromRange:range documentAttributes:nil];
    if ((data == nil) && (outError != NULL)) {
        *outError = [NSError errorWithDomain:NSCocoaErrorDomain code:NSFileWriteUnknownError userInfo:nil];
    }
    return data;
}

- (BOOL)prepareSavePanel:(NSSavePanel *)savePanel { // ADDED IN RECIPE 8
	if (floor(NSAppKitVersionNumber) > NSAppKitVersionNumber10_5) {
		VRDocumentController *controller = [VRDocumentController sharedDocumentController];
		if (![controller canOpenURL:[controller currentDiaryURL]]) {
			[savePanel setNameFieldStringValue:NSLocalizedString(@"Chef’s Diary", @"default name of Chef's Diary document")];
		}
	}
	return YES;
}

- (BOOL)saveToURL:(NSURL *)absoluteURL ofType:(NSString *)typeName forSaveOperation:(NSSaveOperationType)saveOperation error:(NSError **)outError { // ADDED IN RECIPE 6 (final version)
	BOOL success = [super saveToURL:absoluteURL ofType:typeName forSaveOperation:saveOperation error:outError];
	/* REPLACED IN RECIPE 7:
	if (success && (saveOperation != NSAutosaveOperation)) {
		[[VRDocumentController sharedDocumentController] setCurrentDiaryURL:absoluteURL];
		[[NSNotificationCenter defaultCenter] postNotificationName:VRDidSaveDiaryDocumentNotification object:self]; // ADDED IN RECIPE 7
	}
	*/
	if (success) { // ADDED IN RECIPE 7
		if (saveOperation == NSAutosaveOperation) {
			[[NSNotificationCenter defaultCenter] postNotificationName:VRDidAutosaveDiaryDocumentNotification object:self];
		} else {
			[[VRDocumentController sharedDocumentController] setCurrentDiaryURL:absoluteURL];
			[[NSNotificationCenter defaultCenter] postNotificationName:VRDidSaveDiaryDocumentNotification object:self];
		}
	}
	return success;
}

- (BOOL)readFromData:(NSData *)data ofType:(NSString *)typeName error:(NSError **)outError { // ADDED IN RECIPE 3
    NSTextStorage *textStorage = [[NSTextStorage alloc] initWithRTF:data documentAttributes:NULL];
    if ((textStorage == nil) && (outError != NULL)) {
        *outError = [NSError errorWithDomain:NSCocoaErrorDomain code:NSFileReadUnknownError userInfo:nil];
        return NO;
    }
    [self setDiaryDocTextStorage:textStorage];
    [textStorage release];
    return YES;
}

- (void)saveToURL:(NSURL *)absoluteURL ofType:(NSString *)typeName forSaveOperation:(NSSaveOperationType)saveOperation delegate:(id)delegate didSaveSelector:(SEL)didSaveSelector contextInfo:(void *)contextInfo { // ADDED IN RECIPE 4
	NSNumber *saveOperationNumber = [NSNumber numberWithInt:saveOperation];
	[super saveToURL:absoluteURL ofType:typeName forSaveOperation:saveOperation delegate:self didSaveSelector:@selector(document:didSave:contextInfo:) contextInfo:[saveOperationNumber retain]];
}

- (void)document:(NSDocument *)doc didSave:(BOOL)didSave contextInfo:(void *)contextInfo { // ADDED IN RECIPE 4
	// didSaveSelector callback selector for saveDocumentWithDelegate:didSaveSelector:contextInfo:.
	NSSaveOperationType saveOperation = [(NSNumber *)contextInfo intValue];
	[(NSNumber *)contextInfo release];
	if (didSave && (saveOperation != NSAutosaveOperation)) {
		for (NSLayoutManager *manager in [[self diaryDocTextStorage] layoutManagers]) {
			for (NSTextContainer *container in [manager textContainers]) {
				[[container textView] breakUndoCoalescing];
			}
		}
	}
}

- (BOOL)keepBackupFile { // ADDED IN RECIPE 7
	return YES;
}

- (BOOL)revertToContentsOfURL:(NSURL *)absoluteURL ofType:(NSString *)typeName error:(NSError **)outError { // ADDED IN RECIPE 7
	[self setIsRevertingToSavedDocument:YES];
	BOOL success = [super revertToContentsOfURL:absoluteURL ofType:typeName error:outError];
	[self setIsRevertingToSavedDocument:NO];
	return success;
}

- (BOOL)readFromURL:(NSURL *)absoluteURL ofType:(NSString *)typeName error:(NSError **)outError { // ADDED IN RECIPE 7
	// Called from -[NSDocument revertToContentsOfURL:ofType:error:] whenever the user chooses the Revert to Saved menu item.
	if ([self isRevertingToSavedDocument]) {
		NSMutableAttributedString *emptyString = [[[NSMutableAttributedString alloc] initWithString:@""] autorelease];
		[[self diaryDocTextStorage] setAttributedString:emptyString];
		NSError *error;
		BOOL success = [[self diaryDocTextStorage] readFromURL:absoluteURL options:nil documentAttributes:nil error:&error];
		if (!success) [self presentError:error];
		return success;
	} else {
		return [super readFromURL:absoluteURL ofType:typeName error:outError];
	}
}

- (NSPrintOperation *)printOperationWithSettings:(NSDictionary *)printSettings error:(NSError **)outError { // ADDED IN RECIPE 8
	// Called indirectly from the -saveDocumentAsPDFTo: action method to save the document as a PDF file.
	NSPrintInfo *tempPrintInfo = [self printInfo];
	if ([printSettings count] > 0) {
		tempPrintInfo = [[tempPrintInfo copy] autorelease];
		[[tempPrintInfo dictionary] addEntriesFromDictionary:printSettings];
	}
	NSPrintOperation *op = [NSPrintOperation printOperationWithView:[[[self windowControllers] objectAtIndex:0] keyDiaryView] printInfo:tempPrintInfo];
    if ((op == nil) && (outError != NULL)) {
        *outError = [NSError errorWithDomain:NSCocoaErrorDomain code:NSUserCancelledError userInfo:nil];
    }
	return op;
}

#pragma mark MENU ITEM VALIDATION

- (BOOL)validateUserInterfaceItem:(id <NSValidatedUserInterfaceItem>)item { // ADDED IN RECIPE 7
	// Protocol method per NSUserInterfaceValidations protocol. Called automatically by -[NSMenu validateMenuItem:].
	SEL action = [item action];
	if (action == @selector(revertDocumentToSaved:)) {
		VRDocumentController *controller = [VRDocumentController sharedDocumentController];
		NSURL *currentDiaryURL = [controller currentDiaryURL];
		return [controller canOpenURL:currentDiaryURL] && [self isDocumentEdited];
	}
	return [super validateUserInterfaceItem:item];
}

#pragma mark ENTRY AND TAG TITLES

- (NSString *)entryMarker { // ADDED IN RECIPE 4
	return DIARY_TITLE_MARKER;
}

- (NSString *)entryTitleForDate:(NSDate *)date { // ADDED IN RECIPE 4
	NSString *dateString;
	
	// Convert date and time to formatted string.
	if (floor(NSFoundationVersionNumber) <= NSFoundationVersionNumber10_5) {
		NSDateFormatter *formatter = [[NSDateFormatter alloc] init];
		[formatter setDateStyle:NSDateFormatterLongStyle];
		[formatter setTimeStyle:NSDateFormatterLongStyle];
		dateString = [formatter stringFromDate:date];
		[formatter release];
	} else {
		dateString = [NSDateFormatter localizedStringFromDate:date dateStyle:NSDateFormatterLongStyle timeStyle:NSDateFormatterLongStyle];
	}
	return dateString;
}

- (NSString *)entryTitleInsertForDate:(NSDate *)date { // ADDED IN RECIPE 4
	NSTextStorage *storage = [self diaryDocTextStorage];
	NSString *leadingWhitespaceString;
	if ([storage length] == 0) {
		leadingWhitespaceString = @"";
	} else if ([[storage string] hasSuffix:@"\n"]) {
		leadingWhitespaceString = @"\n";
	} else {
		leadingWhitespaceString = @"\n\n";
	}
	NSString *formatString = @"%@%@ %@\n";
	return [NSString stringWithFormat:formatString, leadingWhitespaceString, [self entryMarker], [self entryTitleForDate:date]];
}

- (NSString *)tagMarker { // ADDED IN RECIPE 4
	return DIARY_TAG_MARKER;
}

- (NSString *)tagTitle { // ADDED IN RECIPE 4
	return NSLocalizedString(@"Tags", @"label for diary window tag string");
}

- (NSString *)tagTitleInsert { // ADDED IN RECIPE 4
	NSString *formatString = @"\n%@ %@: ";
	return [NSString stringWithFormat:formatString, [self tagMarker], [self tagTitle]];
}

- (NSUInteger)adjustedIndexIfMarkerAtIndex:(NSUInteger)index {
	// Utility method called by some of the ..EntryTitleRangeForIndex: and ...EntryTagRangeForIndex: methods to implement the rule that an insertion point at a marker character is considered to be in the current entry.
	return ((index < [[self diaryDocTextStorage] length]) && ([[[self diaryDocTextStorage] string] characterAtIndex:index] == [[self entryMarker] characterAtIndex:0])) ? index + 1 : index;
}

- (NSRange)rangeOfLineFromMarkerIndex:(NSUInteger)markerIndex { // ADDED IN RECIPE 4
	// Utility method called by all of the ...EntryTitleRangeForIndex: and ...EntryTagRangeForIndex: methods to complete the operation after the  marker is found by constructing the range to the end of the line.
	if (markerIndex == NSNotFound) return NSMakeRange(NSNotFound, 0);
	NSRange searchRange = NSMakeRange(markerIndex, [[self diaryDocTextStorage] length] - markerIndex);
	NSUInteger newlineIndex = [[[self diaryDocTextStorage] string] rangeOfString:@"\n" options:0 range:searchRange].location;
	if (newlineIndex == NSNotFound) return searchRange;
	return NSMakeRange(markerIndex, newlineIndex - markerIndex); // omit trailing newline
}

- (NSRange)currentEntryTitleRangeForIndex:(NSUInteger)index { // ADDED IN RECIPE 4
	// The index argument is normally the insertion point, but it can be any location in the text storage. If there is no entry marker preceding index, returns range with location of NSNotFound and length of 0, indicating insertion point is in untitled preface and there is no current entry. If there is an entry marker preceding index, returns range with location of preceding entry marker and length of title excluding any trailing newline, indicating insertion point is in current entry. Does not change current insertion point or selection in text view. If index is at an entry marker, it is considered to be in the current entry.
	// Search backward for preceding entry marker or beginning of file.
	if ([[self diaryDocTextStorage] length] == 0) return NSMakeRange(NSNotFound, 0);
	index = [self adjustedIndexIfMarkerAtIndex:index];
	NSUInteger markerIndex = [[[self diaryDocTextStorage] string] rangeOfString:[self entryMarker] options:NSBackwardsSearch range:NSMakeRange(0, index)].location;
	return [self rangeOfLineFromMarkerIndex:markerIndex];
}

- (NSRange)firstEntryTitleRange { // ADDED IN RECIPE 4
	// Called by -[DiaryWindowController goToFirstEntry:] action method. Returns the range of the first entry title. The range is {NSNotfound, 0} if no entry marker is found.	
	// Search forward from beginning of file for entry marker or end of file.
	if ([[self diaryDocTextStorage] length] == 0) return NSMakeRange(NSNotFound, 0);
	NSUInteger markerIndex = [[[self diaryDocTextStorage] string] rangeOfString:[self entryMarker] options:0 range:NSMakeRange(0, [[self diaryDocTextStorage] length])].location;
	return [self rangeOfLineFromMarkerIndex:markerIndex];
}

- (NSRange)lastEntryTitleRange { // ADDED IN RECIPE 4
	// Called by -[DiaryWindowController goToLastEntry:] action method. Returns the range of the last entry title. The range is {NSNotfound, 0} if no entry marker is found.	
	// Search backward from end of file for entry marker or beginning of file.
	if ([[self diaryDocTextStorage] length] == 0) return NSMakeRange(NSNotFound, 0);
	NSUInteger markerIndex = [[[self diaryDocTextStorage] string] rangeOfString:[self entryMarker] options:NSBackwardsSearch range:NSMakeRange(0, [[self diaryDocTextStorage] length])].location;
	return [self rangeOfLineFromMarkerIndex:markerIndex];
}

- (NSRange)previousEntryTitleRangeForIndex:(NSUInteger)index { // ADDED IN RECIPE 4
	// Called by -[DiaryWindowController goToPreviousEntry:] action method. Returns the range of the entry title before the current entry's title. The range is {NSNotfound, 0} if no entry marker is found before the current entry's marker.
	// Search backward for preceding entry marker or beginning of file.
	NSRange startRange = [self currentEntryTitleRangeForIndex:index];
	if (startRange.location == NSNotFound) return NSMakeRange(NSNotFound, 0); // no entry marker found
	
	// Search backward from preceding entry marker for next preceding entry marker or beginning of file.
	NSUInteger markerIndex = [[[self diaryDocTextStorage] string] rangeOfString:[self entryMarker] options:NSBackwardsSearch range:NSMakeRange(0, startRange.location)].location;
	return [self rangeOfLineFromMarkerIndex:markerIndex];
}

- (NSRange)nextEntryTitleRangeForIndex:(NSUInteger)index { // ADDED IN RECIPE 4
	// The index argument is normally the insertion point, but it can be any location in the text storage. If there is no entry marker following index, returns range with location of NSNotFound and length of 0 indicating there is no next entry. If there is an entry marker following index, returns range with location of following entry marker and length of title excluding any trailing newline. Does not change current insertion point or selection in text view. If index is at an entry marker, it is considered to be in the current entry.
	// Search forward for following entry marker or end of file.
	if ([[self diaryDocTextStorage] length] == 0) return NSMakeRange(NSNotFound, 0);
	index = [self adjustedIndexIfMarkerAtIndex:index];
	NSUInteger markerIndex = [[[self diaryDocTextStorage] string] rangeOfString:[self entryMarker] options:0 range:NSMakeRange(index, [[self diaryDocTextStorage] length] - index)].location;
	return [self rangeOfLineFromMarkerIndex:markerIndex];
}

- (NSRange)firstEntryTitleRangeAtOrAfterDate:(NSDate *)targetDate { // ADDED IN RECIPE 4
	// Called by -[DiaryWindowController goToDatedEntry:] action method. Returns the range of the first entry title whose date is after targetDate. The range is {NSNotfound, 0} if no later entry marker is found.
	NSDate *date;
	NSRange tempRange;
	NSRange returnRange = [self firstEntryTitleRange];
	if (returnRange.location == NSNotFound) return NSMakeRange(NSNotFound, 0); // no entry marker found
	do {
		date = [self dateFromEntryTitleRange:returnRange];
		tempRange = [self nextEntryTitleRangeForIndex:returnRange.location + returnRange.length];
		if (tempRange.location == NSNotFound) break; // no more entry markers found
		if ((date == nil) || ([date timeIntervalSinceDate:targetDate] < 0)) returnRange = tempRange;
	} while ((date == nil) || ([date timeIntervalSinceDate:targetDate] < 0));
	return returnRange;
}

- (NSDate *)dateFromEntryTitleRange:(NSRange)range { // ADDED IN RECIPE 4
	// Utility method converts the text in range to an NSDate object, returning nil on failure. The range argument should be an entry title range.
	if ((range.location == NSNotFound) || (range.length < 2)) return nil;
	range.location += 2;
	range.length -= 2;
	NSString *dateString = [[[self diaryDocTextStorage] string] substringWithRange:range];
	NSDateFormatter *formatter = [[NSDateFormatter alloc] init];
	[formatter setDateStyle:NSDateFormatterLongStyle];
	[formatter setTimeStyle:NSDateFormatterLongStyle];
	NSDate *date = [formatter dateFromString:dateString];
	[formatter release];
	return date;
}

- (NSRange)currentEntryTagRangeForIndex:(NSUInteger)index { // ADDED IN RECIPE 4
	// The index argument is normally the insertion point, but it can be any location in the text storage. If index is in untitled preface, returns range with location of NSNotFound and length of 0 indicating tags are illegal in untitled preface. If index is in current entry and current entry has no tag marker, returns range with location where tag marker should be inserted (at end of title excluding any trailing newline) with length of 0. If current entry already has a tag marker, returns range with location where user should start typing tags (at end of tag line excluding any trailing newline) with length of 0. Does not change current insertion point or selection in text view. If index is at an entry marker, it is considered to be in the current entry.
	
	// Search backward for preceding entry marker or beginning of file.
	NSRange startRange = [self currentEntryTitleRangeForIndex:index];
	if (startRange.location == NSNotFound) return NSMakeRange(NSNotFound, 0); // no entry marker found
	
	// Search forward from end of preceding title for following entry marker or end of file.
	NSUInteger tagMarkerInsertionIndex = startRange.location + startRange.length;
	NSRange endRange = [self nextEntryTitleRangeForIndex:tagMarkerInsertionIndex];
	if (endRange.location == NSNotFound) endRange.location = [[self diaryDocTextStorage] length];
	
	// Search forward from end of preceding title to following title marker or end of file for tag marker.
	NSUInteger markerIndex = [[[self diaryDocTextStorage] string] rangeOfString:[self tagMarker] options:0 range:NSMakeRange(tagMarkerInsertionIndex, endRange.location - tagMarkerInsertionIndex)].location;
	if (markerIndex == NSNotFound) return NSMakeRange(tagMarkerInsertionIndex, 0);
	
	return [self rangeOfLineFromMarkerIndex:markerIndex];
}

- (NSArray *)foundTagRangeArrayForTag:(NSString *)tag { // ADDED IN RECIPE 4
	if ([tag length] > 0) {
		NSString *tagLabel = [NSString stringWithFormat:NSLocalizedString(@"%@ Tags: ", @"search string for diary document tag string"), [self tagMarker]];
		NSMutableArray *tagRangeArray = [NSMutableArray array];
		NSRange searchRange = [self firstTagRange];
		NSRange foundRange;
		while (searchRange.location != NSNotFound) {
			searchRange.location += [tagLabel length];
			searchRange.length -= [tagLabel length];
			foundRange = [[[self diaryDocTextStorage] string] rangeOfString:tag options:0 range:searchRange];
			if (foundRange.location != NSNotFound) [tagRangeArray addObject:[NSValue valueWithRange:foundRange]];
			searchRange = [self nextTagRangeForIndex:searchRange.location + searchRange.length];
		}
		return [[tagRangeArray copy] autorelease];
	}
	return nil;
}

- (NSRange)firstTagRange { // ADDED IN RECIPE 4
	// Returns the range of the first tag line, which might not be in the first entry because entries are not required to have tags. The range is {NSNotfound, 0} if no tag marker is found.	
	// Search forward from beginning of file for tag marker or end of file.
	if ([[self diaryDocTextStorage] length] == 0) return NSMakeRange(NSNotFound, 0);
	NSUInteger markerIndex = [[[self diaryDocTextStorage] string] rangeOfString:[self tagMarker] options:0 range:NSMakeRange(0, [[self diaryDocTextStorage] length])].location;
	return [self rangeOfLineFromMarkerIndex:markerIndex];
}

- (NSRange)nextTagRangeForIndex:(NSUInteger)index { // ADDED IN RECIPE 4
	// Returns the range of the first tag line following index. The range is {NSNotfound, 0} if no tag marker is found following index.
	// Search forward for following tag marker or end of file.
	if ([[self diaryDocTextStorage] length] == 0) return NSMakeRange(NSNotFound, 0);
	NSUInteger markerIndex = [[[self diaryDocTextStorage] string] rangeOfString:[self tagMarker] options:0 range:NSMakeRange(index, [[self diaryDocTextStorage] length] - index)].location;
	return [self rangeOfLineFromMarkerIndex:markerIndex];
}

@end

NSString *VRDidSaveDiaryDocumentNotification = @"did save diary document notification"; // ADDED IN RECIPE 7
NSString *VRDidAutosaveDiaryDocumentNotification = @"did autosave diary document notification"; // ADDED IN RECIPE 7
