//
//  DiaryWindowController.m
//  Vermont Recipes 2.0.0
//
//  Created by Bill Cheeseman on 2009-06-05.
//  Copyright 2000-2009 Bill Cheeseman. All rights reserved.
//
//  CREATED IN RECIPE 3

#import "DiaryWindowController.h"
#import "DiaryDocument.h" // ADDED IN RECIPE 3
#import "VRDocumentController.h" // ADDED IN RECIPE 7

#pragma mark MACROS

#define DEFAULT_ALERT_RESTORE_DIARY_DOCUMENT_SUPPRESSED_KEY	@"alert restore diary document suppressed" // ADDED IN RECIPE 7

@implementation DiaryWindowController

#pragma mark INITIALIZATION

- (id)init { // ADDED IN RECIPE 3
	if ((self = [self initWithWindowNibName:@"DiaryWindow"])) {
		// Insert any initialization code here.
	}
	return self;
}

#pragma mark ACCESSOR METHODS

- (NSTextView *)diaryView { // ADDED IN RECIPE 3
	return [[diaryView retain] autorelease];
}

- (NSTextView *)otherDiaryView { // ADDED IN RECIPE 3
	return [[otherDiaryView retain] autorelease];
}

- (NSDatePicker *)datePicker { // ADDED IN RECIPE 4
	return [[datePicker retain] autorelease];
}

- (NSSearchField *)searchField { // ADDED IN RECIPE 5
	return [[searchField retain] autorelease];
}

- (NSSplitView *)splitView { // ADDED IN RECIPE 7
	return [[splitView retain] autorelease];
}

- (NSButton *)addTagButton { // ADDED IN RECIPE 8
	return [[addTagButton retain] autorelease];
}

- (void)setEventMonitor:(id)monitor { // ADDED IN RECIPE 8
	eventMonitor = monitor;
}

- (id)eventMonitor { // ADDED IN RECIPE 8
	return eventMonitor;
}

- (void)setDidSaveDiaryDocumentObserver:(id)observer { // ADDED IN RECIPE 8
	didSaveDiaryDocumentObserver = observer;
}

- (id)didSaveDiaryDocumentObserver { // ADDED IN RECIPE 8
	return didSaveDiaryDocumentObserver;
}

- (void)setDidAutosaveDiaryDocumentObserver:(id)observer { // ADDED IN RECIPE 8
	didAutosaveDiaryDocumentObserver = observer;
}

- (id)didAutosaveDiaryDocumentObserver { // ADDED IN RECIPE 8
	return didAutosaveDiaryDocumentObserver;
}

- (void)setDidRestoreAutosavedDiaryDocumentObserver:(id)observer { // ADDED IN RECIPE 8
	didRestoreAutosavedDiaryDocumentObserver = observer;
}

- (id)didRestoreAutosavedDiaryDocumentObserver { // ADDED IN RECIPE 8
	return didRestoreAutosavedDiaryDocumentObserver;
}

#pragma mark ACTION METHODS

- (IBAction)addEntry:(id)sender { // ADDED IN RECIPE 4
	// Appends new entry title consisting of entry marker and current date at end of diary, with trailing newline. Moves insertion point to beginning of next line after new entry title (at end of text storage) and scrolls it into view so user can start typing.
	NSTextView *keyView = [self keyDiaryView];
	NSTextStorage *storage = [keyView textStorage];
	NSString *titleString = [[self document] entryTitleInsertForDate:[NSDate date]];
	
	// Append entry title and newline to end of diary, supporting undo, notifications and delegates.
	NSRange endRange = NSMakeRange([storage length], 0);
	if ([keyView shouldChangeTextInRange:endRange replacementString:titleString]) {
		[storage replaceCharactersInRange:endRange withString:titleString];
		endRange.length = [titleString length] - 1; // skip trailing newline
		[storage applyFontTraits:NSBoldFontMask range:endRange];
		[keyView didChangeText];
		[[keyView undoManager] setActionName:NSLocalizedString(@"Add Entry", @"name of undo action for Add Entry")];
		
		// Scroll entry title into view and set insertion point to begin typing.
		[[self window] makeFirstResponder:keyView];
		[keyView scrollRangeToVisible:endRange];
		[keyView setSelectedRange:NSMakeRange(endRange.location + endRange.length + 1, 0)]; // following trailing newline
	}
}

- (IBAction)addTag:(id)sender { // ADDED IN RECIPE 4
	// If insertion point is in an entry that has no tag title, inserts new tag title consisting of tag marker and "Tags: " after current entry's title, without trailing newline. Moves insertion point to end of new or existing tag title and scrolls it into view so user can start typing.
	NSTextView *keyView = [self keyDiaryView];
	NSTextStorage *storage = [keyView textStorage];
	NSString *tagString = [[self document] tagTitleInsert];
	
	// Insert tag title after current entry's title, if any, supporting undo, notifications and delegates.
	NSRange tagRange = [[self document] currentEntryTagRangeForIndex:[self insertionPointIndex]];
	if (tagRange.location == NSNotFound) return; // insertion point is in untitled preface
	if (tagRange.length == 0) { // current entry has no existing tag title
		if ([keyView shouldChangeTextInRange:tagRange replacementString:tagString]) {
			[storage replaceCharactersInRange:tagRange withString:tagString];
			tagRange.length = [tagString length];
			[storage applyFontTraits:NSUnboldFontMask range:tagRange];
			[keyView didChangeText];
			[[keyView undoManager] setActionName:NSLocalizedString(@"Add Tag", @"name of undo action and button for Add Tag")];
		}
	}
	
	// Scroll tag title into view and set insertion point to begin typing.
	[[self window] makeFirstResponder:keyView];
	[keyView scrollRangeToVisible:tagRange];
	[keyView setSelectedRange:NSMakeRange(tagRange.location + tagRange.length, 0)];
}

- (IBAction)tagAll:(id)sender { // ADDED IN RECIPE 8
	// Scan diary from top to bottom looking for entries without an associated tag line, and add a tag line to each. Moves insertion point to end of last new or existing tag title and scrolls it into view so user can start typing.
	NSTextView *keyView = [self keyDiaryView];
	NSTextStorage *storage = [keyView textStorage];
	NSString *tagString = [[self document] tagTitleInsert];
	
	// Insert tag title after each entry's title for entries that don't already have tag titles.
	NSRange thisEntryRange = [[self document] firstEntryTitleRange];
	if (thisEntryRange.location == NSNotFound) return;
	NSRange thisTagRange;
	do {
		thisTagRange = [[self document] currentEntryTagRangeForIndex:thisEntryRange.location + thisEntryRange.length];
		if (thisTagRange.length == 0) { // entry has no existing tag title
			if ([keyView shouldChangeTextInRange:thisTagRange replacementString:tagString]) {
				[storage replaceCharactersInRange:thisTagRange withString:tagString];
				thisTagRange.length = [tagString length];
				[storage applyFontTraits:NSUnboldFontMask range:thisTagRange];
				[keyView didChangeText];
				[[keyView undoManager] setActionName:NSLocalizedString(@"Tag All", @"name of undo action, menu item, and button for Tag All")];
			}
		}
		thisEntryRange = [[self document] nextEntryTitleRangeForIndex:thisEntryRange.location + thisEntryRange.length];
	} while (thisEntryRange.location != NSNotFound);
	
	// Scroll last tag title into view and set insertion point to begin typing.
	[[self window] makeFirstResponder:keyView];
	[keyView scrollRangeToVisible:thisTagRange];
	[keyView setSelectedRange:NSMakeRange(thisTagRange.location + thisTagRange.length, 0)];
}

- (IBAction)goToFirstEntry:(id)sender { // ADDED IN RECIPE 4
	// Action method for first entry navigation button.
	NSTextView *keyView = [self keyDiaryView];
	NSRange targetRange = [[self document] firstEntryTitleRange];
	if (targetRange.location != NSNotFound) {
		[[self window] makeFirstResponder:keyView];
		[keyView scrollRangeToVisible:targetRange];
		[keyView setSelectedRange:targetRange];
	}
}

- (IBAction)goToLastEntry:(id)sneder { // ADDED IN RECIPE 4
	// Action method for last entry navigation button.
	NSTextView *keyView = [self keyDiaryView];
	NSRange targetRange = [[self document] lastEntryTitleRange];
	if (targetRange.location != NSNotFound) {
		[[self window] makeFirstResponder:keyView];
		[keyView scrollRangeToVisible:targetRange];
		[keyView setSelectedRange:targetRange];
	}
}

 - (IBAction)goToPreviousEntry:(id)sender { // ADDED IN RECIPE 4
	 // Action method for previous entry navigation button.
	 NSTextView *keyView = [self keyDiaryView];
	 NSRange targetRange = [[self document] previousEntryTitleRangeForIndex:[self insertionPointIndex]];
	 if (targetRange.location != NSNotFound) {
		 [[self window] makeFirstResponder:keyView];
		 [keyView scrollRangeToVisible:targetRange];
		 [keyView setSelectedRange:targetRange];
	 }
 }

- (IBAction)goToNextEntry:(id)sender { // ADDED IN RECIPE 4
	// Action method for next entry navigation button.
	NSTextView *keyView = [self keyDiaryView];
	NSRange targetRange = [[self document] nextEntryTitleRangeForIndex:[self insertionPointIndex]];
	if (targetRange.location != NSNotFound) {
		[[self window] makeFirstResponder:keyView];
		[keyView scrollRangeToVisible:targetRange];
		[keyView setSelectedRange:targetRange];
	}
}

- (IBAction)goToDatedEntry:(id)sender { // ADDED IN RECIPE 4
	// Action method for date picker. Because the date picker must remain in control while the user is changing its settings, this method does not make the text view first responder as the other -goTo... methods do.
	NSTextView *keyView = [self keyDiaryView];
	NSRange targetRange = [[self document] firstEntryTitleRangeAtOrAfterDate:[sender dateValue]];
	if (targetRange.location != NSNotFound) {
		[keyView scrollRangeToVisible:targetRange];
		[keyView setSelectedRange:targetRange];
	}
}

- (IBAction)findTag:(id)sender { // ADDED IN RECIPE 4
	// The action method for the search field. Selects all found tags in valid tag strings, then scrolls to the first found tag and highlights it briefly. Entering the same search string repeatedly cycles highlighting among all found tags.
	if ([sender isKindOfClass:[NSMenuItem class]]) { // ADDED IN RECIPE 5
		[[self window] makeFirstResponder:[self searchField]];
	} else {
		NSTextView *keyView = [self keyDiaryView];
		NSString *tagString = [sender stringValue];
		NSArray *tagRangeArray = [[self document] foundTagRangeArrayForTag:tagString];
		if ([tagRangeArray count] > 0) {
			[keyView setSelectedRanges:tagRangeArray];
			static NSUInteger highlightIndex = 0;
			static NSUInteger tagStringLength = 0;
			if (tagStringLength != [tagString length]) highlightIndex = 0;
			NSRange highlightRange = [[tagRangeArray objectAtIndex:highlightIndex] rangeValue];
			tagStringLength = [tagString length];
			highlightIndex = (highlightIndex < [tagRangeArray count] - 1) ? highlightIndex + 1 : 0;
			[keyView scrollRangeToVisible:highlightRange];
			[keyView showFindIndicatorForRange:highlightRange];
		}
	}
}

#pragma mark OVERRIDE METHODS

- (void)windowDidLoad { // ADDED IN RECIPE 3
	
	// Fix nib file issues involving new features in Snow Leopard. ADDED IN RECIPE 8
	if (floor(NSAppKitVersionNumber) > NSAppKitVersionNumber10_5) {
		[[[self searchField] cell] setUsesSingleLineMode:YES];
		[[self splitView] setDividerStyle:NSSplitViewDividerStylePaneSplitter];
	}
	
	// Set up the diary document text storage for all panes.
	NSTextStorage *docTextStorage = [[self document] diaryDocTextStorage];
	if (docTextStorage == nil) {
		[[self document] setDiaryDocTextStorage:[[self diaryView] textStorage]];
	} else {
		[[[self diaryView] layoutManager] replaceTextStorage:docTextStorage];
	}		
	[[[self otherDiaryView] layoutManager] replaceTextStorage:[[self diaryView] textStorage]];
	
	// Save size of initial standard state for window to user defaults for use when zooming. ADDED IN RECIPE 7
	/*
	 NSSize standardSize = [[self window] frame].size;
	 NSData *standardSizeData = [NSData dataWithBytes:&standardSize length:sizeof(NSSize)];
	 [[NSUserDefaults standardUserDefaults] setObject:standardSizeData forKey:VRDefaultDiaryWindowStandardSizeKey];
	 */
	NSSize standardSize = [[self window] frame].size;
	NSString *standardSizeString = NSStringFromSize(standardSize);
	[[NSUserDefaults standardUserDefaults] setObject:standardSizeString forKey:VRDefaultDiaryWindowStandardSizeKey];
	
	// Set window UI of saved document to autosave state. ADDED IN RECIPE 7
	VRDocumentController *controller = [VRDocumentController sharedDocumentController];
	if ([controller canOpenURL:[controller currentDiaryURL]] || [[self document] isRestoringAutosavedDocument]) {
		[[self window] setFrameAutosaveName:VRDiaryWindowAutosaveName];
		[[self splitView] setAutosaveName:VRDiarySplitViewAutosaveName];
		[[self document] setIsRestoringAutosavedDocument:NO]; // set in -[DiaryDocument initforURL:withContentsOfURL:ofType:error:]
	}
	
	// Install Option key monitor for Add Tag/Tag All button. ADDED IN RECIPE 8
#if MAC_OS_X_VERSION_MIN_REQUIRED >= MAC_OS_X_VERSION_10_6
	// Blocks-based event monitor API, added in Mac OS X 10.6 Snow Leopard.
	[self setEventMonitor:[NSEvent addLocalMonitorForEventsMatchingMask:NSFlagsChangedMask handler:^(NSEvent *incomingEvent) { // removed in -windowWillClose:
		[self toggleAddTagButtonForModifierFlags:[incomingEvent modifierFlags]];
		return incomingEvent;
	}]];
	for (NSMenuItem *thisMenuItem in [[NSApp mainMenu] itemArray]) {
		[[thisMenuItem submenu] setDelegate:self];
	}
	
	// [[self addTagButton] setToolTip:NSLocalizedString(@"Press Option to tag all entries.", @"help tag text for Add Tag button under Snow Leopard")];
	[[self addTagButton] setToolTip:NSLocalizedString(@"Add tag. Press Option to tag all entries.", @"help tag text for Add Tag button under Snow Leopard")];
	[[[self addTagButton] cell] accessibilitySetOverrideValue:NSLocalizedString(@"Add diary tag. Press Option to tag all entries.", @"accessibility help text for Add Tag button under Snow Leopard") forAttribute:NSAccessibilityHelpAttribute];
#else
	[[self addTagButton] setToolTip:NSLocalizedString(@"Add tag.", @"help tag text for Add Tag button under Leopard")];	
#endif
	
	// Register notification observers for document save, autosave and revert.
	NSNotificationCenter *defaultCenter = [NSNotificationCenter defaultCenter];
#if MAC_OS_X_VERSION_MIN_REQUIRED < MAC_OS_X_VERSION_10_6
	[defaultCenter addObserver:self selector:@selector(didSaveDiaryDocument:) name:VRDidSaveDiaryDocumentNotification object:nil]; // ADDED IN RECIPE 7
	[defaultCenter addObserver:self selector:@selector(didAutosaveDiaryDocument:) name:VRDidAutosaveDiaryDocumentNotification object:nil]; // ADDED IN RECIPE 7
	[defaultCenter addObserver:self selector:@selector(didRestoreAutosavedDiaryDocument:) name:VRDidRestoreAutosavedDiaryDocumentNotification object:nil]; // ADDED IN RECIPE 7
#else
	// ADDED IN RECIPE 8
	[self setDidSaveDiaryDocumentObserver:[defaultCenter addObserverForName:VRDidSaveDiaryDocumentNotification object:nil queue:nil usingBlock:^(NSNotification *notification) {
		[self didSaveDiaryDocument:notification];
	}]];
	[self setDidAutosaveDiaryDocumentObserver:[defaultCenter addObserverForName:VRDidAutosaveDiaryDocumentNotification object:nil queue:nil usingBlock:^(NSNotification *notification) {
		[self didAutosaveDiaryDocument:notification];
	}]];
	[self setDidRestoreAutosavedDiaryDocumentObserver:[defaultCenter addObserverForName:VRDidRestoreAutosavedDiaryDocumentNotification object:nil queue:nil usingBlock:^(NSNotification *notification) {
		[self didRestoreAutosavedDiaryDocument:notification];
	}]];
#endif
}

#pragma mark DELEGATE METHODS

- (void)textDidChange:(NSNotification *)notification { // ADDED IN RECIPE 3
	// Delegate method per NSText Delegate Protocol.
	if ([notification object] == [self diaryView]) {
		[[self otherDiaryView] setNeedsDisplayInRect:[[self otherDiaryView] visibleRect] avoidAdditionalLayout:NO];
	} else {
		[[self diaryView] setNeedsDisplayInRect:[[self diaryView] visibleRect] avoidAdditionalLayout:NO];
	}
}

- (void)windowDidUpdate:(NSNotification *)notification { // ADDED IN RECIPE 4
	// Delegate method per NSWindow.
	[self updateWindow];
}

- (NSRect)windowWillUseStandardFrame:(NSWindow *)window defaultFrame:(NSRect)screenFrame { // ADDED IN RECIPE 7
	// Get windows' standard size from user defaults.
	/*
	 NSSize standardSize;
	 [[[NSUserDefaults standardUserDefaults] objectForKey:VRDefaultDiaryWindowStandardSizeKey] getBytes:&standardSize length:sizeof(NSSize)];
	*/
	NSString *standardSizeString = [[NSUserDefaults standardUserDefaults] stringForKey:VRDefaultDiaryWindowStandardSizeKey];
	NSSize standardSize = NSSizeFromString(standardSizeString);
	
	// Adjust window's origin for change to standard height, to anchor top left corner.
	NSRect frame = [window frame];
	frame.origin.y -= standardSize.height - frame.size.height;
	
	return NSMakeRect(frame.origin.x, frame.origin.y, standardSize.width, standardSize.height);
}

- (void)windowWillClose:(NSNotification *)notification { // ADDED IN RECIPE 7
	if (floor(NSAppKitVersionNumber) > NSAppKitVersionNumber10_5) { // ADDED IN RECIPE 8
		if ([self eventMonitor]) [NSEvent removeMonitor:[self eventMonitor]];
		for (NSMenuItem *thisMenuItem in [[NSApp mainMenu] itemArray]) {
			[[thisMenuItem submenu] setDelegate:nil];
		}
	}
	
	NSNotificationCenter *defaultCenter = [NSNotificationCenter defaultCenter];	
	if (floor(NSFoundationVersionNumber) <= NSFoundationVersionNumber10_5) {
		[defaultCenter removeObserver:self];
	} else { // ADDED IN RECIPE 8
		[defaultCenter removeObserver:[self didSaveDiaryDocumentObserver]];
		[defaultCenter removeObserver:[self didAutosaveDiaryDocumentObserver]];
		[defaultCenter removeObserver:[self didRestoreAutosavedDiaryDocumentObserver]];
	}
}

- (void)menuDidClose:(NSMenu *)menu { // ADDED IN RECIPE 8
	// Delegate method on NSMenu. Force Add Tag button to respond to state of Option key in case Option key state changed while any menu was open.
	if (floor(NSAppKitVersionNumber) > NSAppKitVersionNumber10_5) {
		[self toggleAddTagButtonForModifierFlags:[NSEvent modifierFlags]];
	}
}

#pragma mark NOTIFICATION METHODS

- (void)didSaveDiaryDocument:(NSNotification *)notification { // ADDED IN RECIPE 7
	// Notification was posted in -[DiaryDocument saveToURL:ofType:forSaveOperation:Error:.
	[[self window] setFrameAutosaveName:VRDiaryWindowAutosaveName];
	[[self window] saveFrameUsingName:VRDiaryWindowAutosaveName]; // force save to user defaults
	[[self splitView] setAutosaveName:VRDiarySplitViewAutosaveName];
	[[self splitView] adjustSubviews]; // force save to user defaults
	[[NSUserDefaults standardUserDefaults] synchronize];
}

- (void)didAutosaveDiaryDocument:(NSNotification *)notification { // ADDED IN RECIPE 7
	// Notification was posted in -[DiaryDocument saveToURL:ofType:forSaveOperation:Error:.
	[[self window] setFrameAutosaveName:VRDiaryWindowAutosaveName];
	[[self window] saveFrameUsingName:VRDiaryWindowAutosaveName]; // force save to user defaults
	[[self splitView] setAutosaveName:VRDiarySplitViewAutosaveName];
	[[self splitView] adjustSubviews]; // force save to user defaults
	[[NSUserDefaults standardUserDefaults] synchronize]; // force write to user defaults
}

- (void)didRestoreAutosavedDiaryDocument:(NSNotification *)notification { // ADDED IN RECIPE 7
	// Notification was posted in -[VRDocumentController reopenDocumentForURL:withContentsOfURL:error:].
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	if (![defaults boolForKey:DEFAULT_ALERT_RESTORE_DIARY_DOCUMENT_SUPPRESSED_KEY]) {
		[[self alertDidRestoreAutosavedDiaryDocument] beginSheetModalForWindow:[self window] modalDelegate:self didEndSelector:@selector(alertDidRestoreAutosavedDiaryDocumentDidEnd:returnCode:contextInfo:) contextInfo:NULL];
	}
}

#pragma mark USER INTERFACE VALIDATION

- (void)updateWindow { // ADDED IN RECIPE 4
	// Called by -windowDidUpdate: delegate method.
	
	// Validate (enable or disable) validated controls.
	for (id thisView in [[[self window] contentView] subviews]) {
		if ([thisView conformsToProtocol:@protocol(VRValidatedControl)]) {
			[thisView validate];
		}
	}
	
	// Set date value of date picker.
	if (![[[self window] firstResponder] isKindOfClass:[NSDatePicker class]]) {
		[self updateDatePickerValue];
	}
}

- (BOOL)validateUserInterfaceItem:(id <NSValidatedUserInterfaceItem>)item { // ADDED IN RECIPE 4
	// Protocol method per NSUserInterfaceValidations protocol. Called by -updateWindow and automatically by -[NSMenu validateMenuItem:].
	SEL action = [item action];
	if (action == @selector(addTag:)) {
		return ([[self document] currentEntryTitleRangeForIndex:[self insertionPointIndex]].location != NSNotFound);
	} else if (action == @selector(tagAll:)) { // ADDED IN RECIPE 8
		return ([[self document] firstEntryTitleRange].location != NSNotFound);
	} else if (action == @selector(goToFirstEntry:)) {
		return [[self document] firstEntryTitleRange].location != NSNotFound;
	} else if (action == @selector(goToLastEntry:)) {
		return  [[self document] lastEntryTitleRange].location != NSNotFound;
	} else if (action == @selector(goToPreviousEntry:)) {
		return  [[self document] previousEntryTitleRangeForIndex:[self insertionPointIndex]].location != NSNotFound;
	} else if (action == @selector(goToNextEntry:)) {
		return  [[self document] nextEntryTitleRangeForIndex:[self insertionPointIndex]].location != NSNotFound;
	} else if (action == @selector(goToDatedEntry:)) {
		return  [[self document] firstEntryTitleRange].location != NSNotFound;
		/* REMOVED IN RECIPE 5
		 } else if (action == @selector(findTag:)) {
		 return [[self document] firstTagRange].location != NSNotFound;
		 */
	}
	return YES;
}

- (void)updateDatePickerValue { // ADDED IN RECIPE 4
	// Set date value of date picker. Called by -updateWindow.
	NSRange range = [[self document] currentEntryTitleRangeForIndex:[self insertionPointIndex]];
	if (range.location == NSNotFound) {
		[[self datePicker] setDateValue:[NSDate date]];
	} else {
		[[self datePicker] setDateValue:[[self document] dateFromEntryTitleRange:range]];
	}
}

#pragma mark UTILITY METHODS

- (NSTextView *)keyDiaryView { // ADDED IN RECIPE 4
	return ([[self window] firstResponder] == [self otherDiaryView]) ? [self otherDiaryView] : [self diaryView];
}

- (NSUInteger)insertionPointIndex { // ADDED IN RECIPE 4
	// Insertion point may be equal to text storage length (i.e., 1 greater than last text storage index). For example, when insertion point is at end of text storage having length 1, insertion point is 1 and last text storage index is 0. When text storage has length 0 (empty), insertion point is 0 and last text storage index does not exist (NSNotFound).
	return [[[[self keyDiaryView] selectedRanges] objectAtIndex:0] rangeValue].location;
}

- (void)toggleAddTagButtonForModifierFlags:(NSUInteger)modifierFlags { // ADDED IN RECIPE 8
	// Called only under Snow Leopard from block installed in -windowDidLoad and from -menuDidClose:.
	if ((modifierFlags & NSDeviceIndependentModifierFlagsMask) == NSAlternateKeyMask) {
		[[self addTagButton] setTitle:NSLocalizedString(@"Tag All", @"name of undo action, menu item, and button for Tag All")];
		[[self addTagButton] setAction:@selector(tagAll:)];
		// [[self addTagButton] setToolTip:nil];
		[[self addTagButton] setToolTip:NSLocalizedString(@"Tag all entries.", @"help tag text for Tag All button")];
		[[[self addTagButton] cell] accessibilitySetOverrideValue:NSLocalizedString(@"Tag all diary entries.", @"accessibility tag text for Tag All button") forAttribute:NSAccessibilityHelpAttribute];
	} else {
		[[self addTagButton] setTitle:NSLocalizedString(@"Add Tag", @"name of undo action and button for Add Tag")];
		[[self addTagButton] setAction:@selector(addTag:)];
		// [[self addTagButton] setToolTip:NSLocalizedString(@"Press Option to tag all entries.", @"help tag text for Add Tag button under Snow Leopard")];
		[[self addTagButton] setToolTip:NSLocalizedString(@"Add tag. Press Option to tag all entries.", @"help tag text for Add Tag button under Snow Leopard")];
		[[[self addTagButton] cell] accessibilitySetOverrideValue:NSLocalizedString(@"Add diary tag. Press Option to tag all entries.", @"accessibility help text for Add Tag button under Snow Leopard") forAttribute:NSAccessibilityHelpAttribute];
	}
}

#pragma mark ALERTS

- (NSAlert *)alertDidRestoreAutosavedDiaryDocument { // ADDED IN RECIPE 7
	NSAlert *alert = [[[NSAlert alloc] init] autorelease];
	[alert setMessageText:[NSString stringWithFormat:NSLocalizedString(@"Document “%@” was restored from an autosaved copy.", @"message text for DidRestoreAutosavedDiaryDocument alert"), [[self document] displayName]]];
	[alert setInformativeText:NSLocalizedString(@"Save it before you close it to avoid losing any changes.", @"informative text for DidRestoreAutosavedDiaryDocument alert")];
	[alert setShowsSuppressionButton:YES];
	return alert;
}

- (void)alertDidRestoreAutosavedDiaryDocumentDidEnd:(NSAlert *)alert returnCode:(NSInteger)returnCode contextInfo:(void *)contextInfo { // ADDED IN RECIPE 7
	// alertDidEnd callback for alertDidRestoreAutosavedDiaryDocument.
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	[defaults setBool:[[alert suppressionButton] state] forKey:DEFAULT_ALERT_RESTORE_DIARY_DOCUMENT_SUPPRESSED_KEY];
	[defaults synchronize];}

@end


#pragma mark -

@implementation ValidatedDiaryButton // ADDED IN RECIPE 4

- (void)validate {
	id validator = [NSApp targetForAction:[self action] to:[self target] from:self];
	if ((validator == nil) || ![validator respondsToSelector:[self action]]) {
		[self setEnabled:NO];
	} else if ([validator respondsToSelector:@selector(validateControl:)]) {
		[self setEnabled:[validator validateControl:self]];
	} else if ([validator respondsToSelector:@selector(validateUserInterfaceItem:)]) {
		[self setEnabled:[validator validateUserInterfaceItem:self]];
	} else {
		[self setEnabled:YES];
	}
}

@end


#pragma mark -

@implementation ValidatedDiaryDatePicker // ADDED IN RECIPE 4

- (void)validate {
	id validator = [NSApp targetForAction:[self action] to:[self target] from:self];
	if ((validator == nil) || ![validator respondsToSelector:[self action]]) {
		[self setEnabled:NO];
	} else if ([validator respondsToSelector:@selector(validateControl:)]) {
		[self setEnabled:[validator validateControl:self]];
	} else if ([validator respondsToSelector:@selector(validateUserInterfaceItem:)]) {
		[self setEnabled:[validator validateUserInterfaceItem:self]];
	} else {
		[self setEnabled:YES];
	}
}

@end


#pragma mark -

@implementation ValidatedDiarySearchField // ADDED IN RECIPE 4

- (void)validate {
	id validator = [NSApp targetForAction:[self action] to:[self target] from:self];
	if ((validator == nil) || ![validator respondsToSelector:[self action]]) {
		[self setEnabled:NO];
	} else if ([validator respondsToSelector:@selector(validateControl:)]) {
		[self setEnabled:[validator validateControl:self]];
	} else if ([validator respondsToSelector:@selector(validateUserInterfaceItem:)]) {
		[self setEnabled:[validator validateUserInterfaceItem:self]];
	} else {
		[self setEnabled:YES];
	}
}

@end

NSString *VRDefaultDiaryWindowStandardSizeKey = @"diary window standard size"; // ADDED IN RECIPE 7
NSString *VRDiaryWindowAutosaveName = @"diary window"; // ADDED IN RECIPE 7
NSString *VRDiarySplitViewAutosaveName = @"diary split view"; // ADDED IN RECIPE 7
