//
//  RecipesWindowController.m
//  Vermont Recipes 2.0.0
//
//  Created by Bill Cheeseman on 2009-05-24.
//  Copyright 2000-2009 Bill Cheeseman. All rights reserved.
//
//  CREATED IN RECIPE 1

#import "RecipesWindowController.h"
#import "NSScreen+VRScreenAdditions.h" // ADDED IN RECIPE 7
#import "NSDrawer+VRDrawerAdditions.h" // ADDED IN RECIPE 7

@implementation RecipesWindowController

#pragma mark ACCESSOR METHODS

- (NSSplitView *)verticalSplitView { // ADDED IN RECIPE 8
	return [[verticalSplitView retain] autorelease];
}

#pragma mark OVERRIDE METHODS

- (void)windowDidLoad { // ADDED IN RECIPE 5
	
	// Fix nib file issues involving new features in Snow Leopard. ADDED IN RECIPE 8
	if (floor(NSAppKitVersionNumber) > NSAppKitVersionNumber10_5) {
		[[self verticalSplitView] setDividerStyle:NSSplitViewDividerStylePaneSplitter];
	}	
	
	// Splice drawer into responder chain.
    NSResponder *oldNextResponder = [self nextResponder];
    NSResponder *newNextResponder = [[[self window] drawers] objectAtIndex:0];
    [self setNextResponder:newNextResponder];
	[newNextResponder setNextResponder:oldNextResponder];
	
	// Set maximum window width. ADDED IN RECIPE 7
	NSRect maxVisibleFrame = [[NSScreen VR_screenWithLargestVisibleFrame] visibleFrame];
	CGFloat drawerMaxWidth = [[[[self window] drawers] objectAtIndex:0] VR_drawerMaxWidth];
	[[self window] setMaxSize:NSMakeSize(maxVisibleFrame.size.width - drawerMaxWidth, maxVisibleFrame.size.height)];

	// Save size of initial standard state for window, as set in Interface Builder, to user defaults. ADDED IN RECIPE 7
	NSSize standardSize = [[self window] frame].size;
	NSString *standardSizeString = NSStringFromSize(standardSize);
	[[NSUserDefaults standardUserDefaults] setObject:standardSizeString forKey:VRDefaultRecipesWindowStandardSizeKey];
}

#pragma mark DELEGATE METHODS

- (void)splitView:(NSSplitView *)splitView resizeSubviewsWithOldSize:(NSSize)oldSize { // ADDED IN RECIPE 2
	// Delegate method per NSSplitView.
	if (floor(NSAppKitVersionNumber) <= NSAppKitVersionNumber10_5) {
		NSView *rightPane = [[splitView subviews] objectAtIndex:1];
		NSRect rightFrame = [rightPane frame];
		rightFrame.size.width += [splitView frame].size.width - oldSize.width;
		[rightPane setFrame:rightFrame];
	}
	[splitView adjustSubviews];
}

- (BOOL)splitView:(NSSplitView *)splitView shouldAdjustSizeOfSubview:(NSView *)view { // ADDED IN RECIPE 2
	// Delegate method per NSSplitView; added in Mac OS X 10.6 Snow Leopard.
	if ([splitView isVertical] && (view == [[splitView subviews] objectAtIndex:0])) {
		return NO;
	}
	return YES;
}

- (NSRect)windowWillUseStandardFrame:(NSWindow *)window defaultFrame:(NSRect)newFrame { // ADDED IN RECIPE 7
	// Get windows' standard size from user defaults.
	NSString *standardSizeString = [[NSUserDefaults standardUserDefaults] stringForKey:VRDefaultRecipesWindowStandardSizeKey];
	NSSize standardSize = NSSizeFromString(standardSizeString);
	
	// Adjust window's origin for change to standard height, to anchor top left corner.
	NSRect frame = [window frame];
	frame.origin.y -= standardSize.height - frame.size.height;
	
	return NSMakeRect(frame.origin.x, frame.origin.y, standardSize.width, standardSize.height);
}

@end

NSString *VRDefaultRecipesWindowStandardSizeKey = @"recipes window standard size"; // ADDED IN RECIPE 7
