//
//  VRDocumentController.m
//  Vermont Recipes 2.0.0
//	
//  Created by Bill Cheeseman on 2009-06-06.
//  Copyright 2000-2009 Bill Cheeseman. All rights reserved.
//
//  CREATED IN RECIPE 3

#import "VRDocumentController.h"
#import "DiaryDocument.h" // ADDED IN RECIPE 3

#pragma mark MACROS

#define DIARY_DOCUMENT_IDENTIFIER			@"com.quecheesoftware.vermontrecipes.diary" // ADDED IN RECIPE 3

// ADDED IN RECIPE 6
#define ERROR_NAME_KEY						@"VRErrorNameKey"
#define ERROR_DIARY_DOCUMENT_ALREADY_OPEN	@"diary document already open"
#define ERROR_INCORRECT_DIARY_DOCUMENT		@"incorrect diary document"

#pragma mark -

@implementation VRDocumentController

#pragma mark ACTION METHODS

- (IBAction)newDiaryDocument:(id)sender { // ADDED IN RECIPE 3
	DiaryDocument *diary = [self makeUntitledDocumentOfType:DIARY_DOCUMENT_IDENTIFIER error:NULL];
	// FIXME: Add error handling?
	if (diary != nil) {
		[self addDocument:diary];
		[diary makeWindowControllers];
		[diary showWindows];
	}
}

- (IBAction)openDiaryDocument:(id)sender { // ADDED IN RECIPE 6 (final version)
	// Connected to Open Chef’s Diary menu item in -validateUserInterfaceItem:.
	NSDocument *diaryDocument = [self diaryDocument];
	if (diaryDocument != nil) { // a diary document is already open
		[[[[diaryDocument windowControllers] objectAtIndex:0] window] makeKeyAndOrderFront:sender];
	} else { // a diary document is not open
		NSURL *fileURL = [self currentDiaryURL];
		if (fileURL != nil) {
			NSError *error;
			DiaryDocument *diary = [self makeDocumentWithContentsOfURL:fileURL ofType:DIARY_DOCUMENT_IDENTIFIER error:&error];
			if (diary == nil) {
				[self presentError:error];
			} else {
				[self addDocument:diary];
				[diary makeWindowControllers];
				[diary showWindows];
			}
		}
	}
}

#pragma mark OVERRIDE METHODS

- (id)openDocumentWithContentsOfURL:(NSURL *)absoluteURL display:(BOOL)displayDocument error:(NSError **)outError { // ADDED IN RECIPE 6
	// Called automatically by -[NSDocumentController openDocument:] action method for Open and Open Recent menu items, and by open document ('odoc') Apple event when user opens document in Finder.
	NSDocument *diaryDocument = [self documentForURL:absoluteURL];
	if (diaryDocument != nil) { // this diary document is already open
		[[[[diaryDocument windowControllers] objectAtIndex:0] window] makeKeyAndOrderFront:nil];
		return diaryDocument;
	} else if ([self diaryDocument] != nil) { // another diary document is already open
		if (outError != NULL) {
			// *outError = [NSError errorWithDomain:NSCocoaErrorDomain code:NSUserCancelledError userInfo:nil];
			*outError = [self diaryDocumentAlreadyOpenError];
		}
	} else {
		NSURL *currentDiaryURL = [self currentDiaryURL];
		if ((currentDiaryURL == nil) || [absoluteURL isEqual:currentDiaryURL]) {
			diaryDocument = [super openDocumentWithContentsOfURL:absoluteURL display:displayDocument error:outError];
			if (diaryDocument != nil) {
				if (currentDiaryURL == nil) [self setCurrentDiaryURL:absoluteURL];
				return diaryDocument;
			}
		} else {
			if (outError != NULL) {
				// *outError = [NSError errorWithDomain:NSCocoaErrorDomain code:NSUserCancelledError userInfo:nil];
				*outError = [self incorrectDiaryDocumentErrorForURL:absoluteURL];
			}
		}
	}
	return nil;
}

- (BOOL)reopenDocumentForURL:(NSURL *)absoluteDocumentURL withContentsOfURL:(NSURL *)absoluteDocumentContentsURL error:(NSError **)outError { // ADDED IN RECIPE 7
	// Called automatically when application reopens an autosaved document after a crash.
	BOOL success = [super reopenDocumentForURL:absoluteDocumentURL withContentsOfURL:absoluteDocumentContentsURL error:outError];
	if (success) {
		if ([[self documentForURL:absoluteDocumentContentsURL] isKindOfClass:[DiaryDocument class]]) {
			[[NSNotificationCenter defaultCenter] postNotificationName:VRDidRestoreAutosavedDiaryDocumentNotification object:self];
		}
	}
	return success;
}

#pragma mark USER INTERFACE VALIDATION

/* REMOVED IN RECIPE 6
- (BOOL)validateUserInterfaceItem:(id <NSValidatedUserInterfaceItem>)item { // ADDED IN RECIPE 6 (preliminary version)
	// Protocol method per NSUserInterfaceValidations protocol. Called automatically by -[NSMenu validateMenuItem:].
	SEL action = [item action];
	if ((action == @selector(newDiaryDocument:)) || (action == @selector(openDiaryDocument:))) {
		for (NSDocument *thisDocument in [self documents]) {
			if ([thisDocument isKindOfClass:[DiaryDocument class]]) {
				return NO;
			}
		}
        return YES;
	}
	return [super validateUserInterfaceItem:item];
}
*/

- (BOOL)validateUserInterfaceItem:(id <NSValidatedUserInterfaceItem>)item { // ADDED IN RECIPE 6 (final version)
	// Protocol method per NSUserInterfaceValidations protocol. Called automatically by -[NSMenu validateMenuItem:].
	SEL action = [item action];
	if ((action == @selector(newDiaryDocument:)) || (action == @selector(openDiaryDocument:))) {
		
		NSURL *fileURL = [self currentDiaryURL];
		if ((fileURL != nil) && [self canOpenURL:fileURL]) {
			[(NSMenuItem *)item setTitle:NSLocalizedString(@"Open Chef’s Diary", @"menu item title for Open Chef’s Diary")];
			[(NSMenuItem *)item setAction:@selector(openDiaryDocument:)];
		} else {
			[(NSMenuItem *)item setTitle:NSLocalizedString(@"New Chef’s Diary", @"menu item title for New Chef’s Diary")];
			[(NSMenuItem *)item setAction:@selector(newDiaryDocument:)];
		}
		
		return (([self diaryDocument] == nil) || ([(NSMenuItem *)item action] == @selector(openDiaryDocument:)));
	}
	return [super validateUserInterfaceItem:item];
}

#pragma mark UTILITY METHODS

- (NSDocument *)diaryDocument { // ADDED IN RECIPE 6
	for (NSDocument *thisDocument in [self documents]) {
		if ([thisDocument isKindOfClass:[DiaryDocument class]]) {
			return thisDocument;
		}
	}
	return nil;
}

#pragma mark ALIAS/BOOKMARK MANAGEMENT

- (NSURL *)currentDiaryURL { // ADDED IN RECIPE 6
	NSData *aliasData = [[NSUserDefaults standardUserDefaults] objectForKey:VRDefaultDiaryDocumentAliasDataKey];
	return [self URLFromAliasData:aliasData];
}

- (void)setCurrentDiaryURL:(NSURL *)absoluteURL { // ADDED IN RECIPE 6
	NSData *aliasData = [self aliasDataFromURL:absoluteURL];
	[[NSUserDefaults standardUserDefaults] setObject:aliasData forKey:VRDefaultDiaryDocumentAliasDataKey];
}

- (NSData *)aliasDataFromURL:(NSURL *)fileURL { // ADDED IN RECIPE 6
	if (fileURL == nil) return nil;	
	if (floor(NSFoundationVersionNumber) > NSFoundationVersionNumber10_5) {
		// Return NSURL bookmark data; added in Mac OS X 10.6 Snow Leopard.
		return [fileURL bookmarkDataWithOptions:0 includingResourceValuesForKeys:nil relativeToURL:nil error:NULL];
		// FIXME: Add error handling?
	} else {
		// Return Alias Manager alias record in Leopard.
		FSRef fsRef;
		if (CFURLGetFSRef((CFURLRef)fileURL, &fsRef)) {
			AliasHandle aliasHdl;
			OSErr err = FSNewAlias(NULL, &fsRef, &aliasHdl);
			if ((err == noErr) && (aliasHdl != NULL)) {
				NSData *returnData = [NSData dataWithBytes:*aliasHdl length:GetAliasSize(aliasHdl)];
				DisposeHandle((Handle)aliasHdl);
				return returnData;
			}
		}
		return nil;
	}
}

- (NSURL *)URLFromAliasData:(NSData *)aliasData { // ADDED IN RECIPE 6
	if (aliasData == nil) return nil;	
	NSURL *returnURL = nil;	
	if (floor(NSFoundationVersionNumber) > NSFoundationVersionNumber10_5) {
		// Return NSURL by resolving bookmark data; added in Mac OS X 10.6 Snow Leopard.
		BOOL isStale;
		returnURL = [NSURL URLByResolvingBookmarkData:aliasData options:0 relativeToURL:nil bookmarkDataIsStale:&isStale error:NULL];
		// FIXME: Add error handling?
		if (returnURL != nil) {
			return returnURL;
		}
	}
	
	// Return NSURL by resolving Alias Manager alias record in Leopard, or in Snow Leopard if aliasData contains an Alias Manager alias record.
	AliasHandle aliasHdl = NULL;
	PtrToHand([aliasData bytes], (Handle*)&aliasHdl, [aliasData length]);
	if (aliasHdl != NULL) {
		FSRef fsRef;
		Boolean wasChanged;
		OSErr err = FSResolveAlias(NULL, aliasHdl, &fsRef, &wasChanged);
		DisposeHandle((Handle)aliasHdl);
		if (err == noErr) {
			returnURL = [(NSURL *)CFURLCreateFromFSRef(NULL, &fsRef) autorelease];
		}
	}	
	return returnURL;
}

- (BOOL)canOpenURL:(NSURL *)absoluteURL { // ADDED IN RECIPE 6
	// Returns YES if absoluteURL is not nil and is not in the trash.
	if (absoluteURL != nil) {
		Boolean isInTrash = true;
		DetermineIfPathIsEnclosedByFolder(kOnAppropriateDisk, kTrashFolderType, (const UInt8 *)[[NSFileManager defaultManager] fileSystemRepresentationWithPath:[absoluteURL path]], false, &isInTrash); // if error, isInTrash remains true
		if (floor(NSFoundationVersionNumber) <= NSFoundationVersionNumber10_5) {
			// Use File Manager in Leopard.
			return [[NSFileManager defaultManager] fileExistsAtPath:[absoluteURL path]] && !isInTrash;
		} else {
			// Use -[NSURL checkResourceIsReachableAndReturnError:]; added in Mac OS X 10.6 Snow Leopard.
			return [absoluteURL checkResourceIsReachableAndReturnError:NULL] && !isInTrash;
		}
	}
	return NO;
}

#pragma mark ERROR HANDLING

- (NSError *)diaryDocumentAlreadyOpenError { // ADDED IN RECIPE 6 (final version)
    NSString *errorFailureReason = NSLocalizedString(@"The current Chef’s Diary is already open.", @"description text for diary document already open error");
	NSString *recoverySuggestion = NSLocalizedString(@"Close it and try again to make a different document the current Chef's Diary.", @"recovery suggestion text for diary document already open error");
    NSDictionary *errorInfo = [NSDictionary dictionaryWithObjectsAndKeys:errorFailureReason,NSLocalizedFailureReasonErrorKey, recoverySuggestion, NSLocalizedRecoverySuggestionErrorKey, ERROR_DIARY_DOCUMENT_ALREADY_OPEN, ERROR_NAME_KEY, nil];
    return [NSError errorWithDomain:NSURLErrorDomain code:NSURLErrorCannotOpenFile userInfo:errorInfo];
}

- (NSError *)incorrectDiaryDocumentErrorForURL:(NSURL *)fileURL { // ADDED IN RECIPE 6 (final version)
	// Called from -openDocumentWithContentsOfURL:display:error:.
	NSString *currentDiaryDocumentName = @"";
	NSURL *currentDiaryDocumentURL = [self currentDiaryURL];
	if (currentDiaryDocumentURL != nil) {
		if (floor(NSFoundationVersionNumber) <= NSFoundationVersionNumber10_5) {
			currentDiaryDocumentName = [[currentDiaryDocumentURL path] lastPathComponent];
		} else {
			currentDiaryDocumentName = [currentDiaryDocumentURL lastPathComponent];
		}
	}
	NSString *errorFailureReason = NSLocalizedString(@"It is not the current Chef’s Diary.", @"failure reason text for incorrect diary document error");
	NSString *recoverySuggestion = [NSString stringWithFormat:NSLocalizedString(@"Click Open As New Diary to make this document the current Chef’s Diary and open it. Click Open Current Diary to open the current Chef’s Diary, “%@.”", @"recovery suggestion text for incorrect diary document error"), currentDiaryDocumentName];
	NSArray *recoveryOptions = [NSArray arrayWithObjects:NSLocalizedString(@"Cancel", @"recovery options title for Cancel button for incorrect diary document error"), NSLocalizedString(@"Open Current Diary", @"recovery options title for Open Current Diary button for incorrect diary document error"), NSLocalizedString(@"Open As New Diary", @"recovery options title for Open As New Diary button for incorrect diary document error"), nil];
	id recoveryAttempter = self;	
	NSDictionary *errorInfo = [NSDictionary dictionaryWithObjectsAndKeys:errorFailureReason, NSLocalizedFailureReasonErrorKey, recoverySuggestion, NSLocalizedRecoverySuggestionErrorKey, recoveryOptions, NSLocalizedRecoveryOptionsErrorKey, recoveryAttempter, NSRecoveryAttempterErrorKey, fileURL, NSURLErrorKey, ERROR_INCORRECT_DIARY_DOCUMENT, ERROR_NAME_KEY, nil];
	return [NSError errorWithDomain:NSURLErrorDomain code:NSURLErrorCannotOpenFile userInfo:errorInfo];
}

- (NSError *)willPresentError:(NSError *)error { // ADDED IN RECIPE 6
	// Error handling method per NSDocumentController.
	if ([[error domain] isEqualToString:NSURLErrorDomain]) {
		switch([error code]) {
			case NSURLErrorCannotOpenFile: {
				if ([[[error userInfo] objectForKey:ERROR_NAME_KEY] isEqualToString:ERROR_DIARY_DOCUMENT_ALREADY_OPEN] || [[[error userInfo] objectForKey:ERROR_NAME_KEY] isEqualToString:ERROR_INCORRECT_DIARY_DOCUMENT]) {
					return error;
					// NSError *underlyingError = [[error userInfo] valueForKey:NSUnderlyingErrorKey];
					// if (underlyingError) return underlyingError;
				}
			}
		}
	}
	return [super willPresentError:error];
}

- (BOOL)attemptRecoveryFromError:(NSError *)error optionIndex:(NSUInteger)recoveryOptionIndex { // ADDED IN RECIPE 6
	// Protocol method per NSErrorRecoveryAttempting informal protocol.
	if ([[error domain] isEqualToString:NSURLErrorDomain]) {
		switch([error code]) {
			case NSURLErrorCannotOpenFile: {
				if ([[[error userInfo] objectForKey:ERROR_NAME_KEY] isEqualToString:ERROR_INCORRECT_DIARY_DOCUMENT]) {
					if (recoveryOptionIndex == 1) { // Open Current Diary button
						[self openDiaryDocument:nil];
						return YES;
					} else if (recoveryOptionIndex == 2) { // Open As New Diary button
						[self setCurrentDiaryURL:[[error userInfo] objectForKey:NSURLErrorKey]];
						[self openDiaryDocument:nil];
						return YES;
					}
				}
			}
		}
	}
	return NO;
}

@end

NSString *VRDefaultDiaryDocumentAliasDataKey = @"diary document alias data"; // ADDED IN RECIPE 6
NSString *VRDidRestoreAutosavedDiaryDocumentNotification = @"did restore autosaved diary document notification"; // ADDED IN RECIPE 7
