//
//  PreferencesWindowController.m
//  Vermont Recipes 2.0.0
//
//  Created by Bill Cheeseman on 2009-11-29.
//  Copyright 2000-2009 Bill Cheeseman. All rights reserved.
//
//  CREATED IN RECIPE 10

#import "PreferencesWindowController.h"
#import "DiaryPrintView.h" // ADDED IN RECIPE 10
#import "DiaryWindowController.h" // ADDED IN RECIPE 10
#import "VRDocumentController.h" // ADDED IN RECIPE 10
#import "RecipesWindowController.h" // ADDED IN RECIPE 10
#import "DiaryPrintPanelAccessoryController.h" // ADDED IN RECIPE 10


@implementation PreferencesWindowController

#pragma mark INITIALIZATION

- (void)dealloc { // ADDED IN RECIPE 10
	NSNotificationCenter *defaultCenter = [NSNotificationCenter defaultCenter];	
	if (floor(NSFoundationVersionNumber) <= NSFoundationVersionNumber10_5) {
		[defaultCenter removeObserver:self];
	} else {
		[defaultCenter removeObserver:[self userDefaultsDidChangeObserver]];
	}
	[super dealloc];
}

#pragma mark FACTORY METHOD

+ (PreferencesWindowController *)sharedController { // ADDED IN RECIPE 10
	static PreferencesWindowController *sharedController = nil;
	if (sharedController == nil) {
		sharedController = [[self alloc] initWithWindowNibName:@"PreferencesWindow"];
	}
	return sharedController;
}

#pragma mark ACCESSOR METHODS

- (void)setUserDefaultsDidChangeObserver:(id)observer { // ADDED IN RECIPE 10
	userDefaultsDidChangeObserver = observer;
}

- (id)userDefaultsDidChangeObserver { // ADDED IN RECIPE 10
	return userDefaultsDidChangeObserver;
}

// General pane.

- (NSButton *)suppressScaleUpAlertCheckbox { // ADDED IN RECIPE 10
	return suppressScaleUpAlertCheckbox;
}

- (NSButton *)suppressRestoreAutosavedDocumentAlertCheckbox { // ADDED IN RECIPE 10
	return suppressRestoreAutosavedDocumentAlertCheckbox;
}

// Recipes pane.

- (NSTextField *)recipesWindowWidthTextField { // ADDED IN RECIPE 10
	return recipesWindowWidthTextField;
}

- (NSTextField *)recipesWindowHeightTextField { // ADDED IN RECIPE 10
	return recipesWindowHeightTextField;
}

- (NSStepper *)recipesWindowWidthStepper { // ADDED IN RECIPE 10
	return recipesWindowWidthStepper;
}

- (NSStepper *)recipesWindowHeightStepper { // ADDED IN RECIPE 10
	return recipesWindowHeightStepper;
}

// Chef's Diary pane.

- (NSTextField *)diaryWindowWidthTextField { // ADDED IN RECIPE 10
	return diaryWindowWidthTextField;
}

- (NSTextField *)diaryWindowHeightTextField { // ADDED IN RECIPE 10
	return diaryWindowHeightTextField;
}

- (NSStepper *)diaryWindowWidthStepper { // ADDED IN RECIPE 10
	return diaryWindowWidthStepper;
}

- (NSStepper *)diaryWindowHeightStepper { // ADDED IN RECIPE 10
	return diaryWindowHeightStepper;
}

- (NSButton *)printTagsCheckbox { // ADDED IN RECIPE 10
	return printTagsCheckbox;
}

- (NSButton *)printHeadersAndFootersCheckbox { // ADDED IN RECIPE 10
	return printHeadersAndFootersCheckbox;
}

- (NSMatrix *)printTimestampRadioGroup { // ADDED IN RECIPE 10
	return printTimestampRadioGroup;
}

- (NSPopUpButton *)diaryAutosaveIntervalButton  { // ADDED IN RECIPE 10
	return diaryAutosaveIntervalButton ;
}

- (NSTextField *)currentDiaryDocumentTextField { // ADDED IN RECIPE 10
	return currentDiaryDocumentTextField;
}

#pragma mark ACTION METHODS

// General pane.

- (IBAction)setDefaultSuppressScaleUpAlert:(id)sender { // ADDED IN RECIPE 10
	[[NSUserDefaults standardUserDefaults] setBool:[sender state] forKey:VRDefaultSuppressScaleUpAlertKey];
}

- (IBAction)setDefaultSuppressRestoreAutosavedDocumentAlert:(id)sender { // ADDED IN RECIPE 10
	[[NSUserDefaults standardUserDefaults] setBool:[sender state] forKey:VRDefaultSuppressRestoreDiaryDocumentAlertKey];
	
	/* Don't do this because it isn't future proof:
	[[[[[[[[VRDocumentController sharedDocumentController] diaryDocument] windowForSheet] attachedSheet] contentView] subviews] objectAtIndex:3] setState:[sender state]];
	*/
}

// Recipes pane.

- (IBAction)setDefaultRecipesWindowWidth:(id)sender { // ADDED IN RECIPE 10
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	NSSize standardSize = NSSizeFromString([defaults stringForKey:VRDefaultRecipesWindowStandardSizeKey]);
	standardSize.width = (CGFloat)[sender integerValue];
	[defaults setObject:NSStringFromSize(standardSize) forKey:VRDefaultRecipesWindowStandardSizeKey];
	
	// Synchronize text field and stepper.
	if ([sender isKindOfClass:[NSStepper class]]) {
		[[self recipesWindowWidthTextField] takeIntegerValueFrom:sender];
	} else {
		[[self recipesWindowWidthStepper] takeIntegerValueFrom:sender];
	}
	
	// Update window if it is currently open and zoomed to old standard size.
	if ([[sender window] isZoomed]) {
		NSWindow *recipesWindow = [[[[[VRDocumentController sharedDocumentController] recipesDocument] windowControllers] objectAtIndex:0] window];
		NSRect newWindowFrame = NSMakeRect([recipesWindow frame].origin.x, [recipesWindow frame].origin.y, standardSize.width, standardSize.height);
		[recipesWindow setFrame:newWindowFrame display:YES];
	}
}

- (IBAction)setDefaultRecipesWindowHeight:(id)sender { // ADDED IN RECIPE 10
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	NSSize standardSize = NSSizeFromString([defaults stringForKey:VRDefaultRecipesWindowStandardSizeKey]);
	standardSize.height = (CGFloat)[sender integerValue];
	[defaults setObject:NSStringFromSize(standardSize) forKey:VRDefaultRecipesWindowStandardSizeKey];
	
	// Synchronize text field and stepper.
	if ([sender isKindOfClass:[NSStepper class]]) {
		[[self recipesWindowHeightTextField] takeIntegerValueFrom:sender];
	} else {
		[[self recipesWindowHeightStepper] takeIntegerValueFrom:sender];
	}
	
	// Update window if it is currently open and zoomed to old standard size.
	if ([[sender window] isZoomed]) {
		NSWindow *recipesWindow = [[[[[VRDocumentController sharedDocumentController] recipesDocument] windowControllers] objectAtIndex:0] window];
		NSRect newWindowFrame = NSMakeRect([recipesWindow frame].origin.x, [recipesWindow frame].origin.y + ([recipesWindow frame].size.height - standardSize.height), standardSize.width, standardSize.height);
		[recipesWindow setFrame:newWindowFrame display:YES];
	}
}

- (IBAction)setDefaultRecipesWindowSizeFromCurrent:(id)sender { // ADDED IN RECIPE 10
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	NSWindow *recipesWindow = [[[[[VRDocumentController sharedDocumentController] recipesDocument] windowControllers] objectAtIndex:0] window];
	NSRect recipesWindowRect = [recipesWindow frame];
	[defaults setObject:NSStringFromSize(recipesWindowRect.size) forKey:VRDefaultRecipesWindowStandardSizeKey];
	
	// Synchronize text fields and steppers.
	[[self recipesWindowWidthTextField] setIntegerValue:recipesWindowRect.size.width];
	[[self recipesWindowHeightTextField] setIntegerValue:recipesWindowRect.size.height];
	[[self recipesWindowWidthStepper] setIntegerValue:recipesWindowRect.size.width];
	[[self recipesWindowHeightStepper] setIntegerValue:recipesWindowRect.size.height];
}

// Chef's Diary pane.

- (IBAction)setDefaultDiaryWindowWidth:(id)sender { // ADDED IN RECIPE 10
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	NSSize standardSize = NSSizeFromString([defaults stringForKey:VRDefaultDiaryWindowStandardSizeKey]);
	standardSize.width = (CGFloat)[sender integerValue];
	[defaults setObject:NSStringFromSize(standardSize) forKey:VRDefaultDiaryWindowStandardSizeKey];
	
	// Synchronize text field and stepper.
	if ([sender isKindOfClass:[NSStepper class]]) {
		[[self diaryWindowWidthTextField] takeIntegerValueFrom:sender];
	} else {
		[[self diaryWindowWidthStepper] takeIntegerValueFrom:sender];
	}
	
	// Update window if it is currently open and zoomed to old standard size.
	if ([[sender window] isZoomed]) {
		NSWindow *diaryWindow = [[[[[VRDocumentController sharedDocumentController] diaryDocument] windowControllers] objectAtIndex:0] window];
		NSRect newWindowFrame = NSMakeRect([diaryWindow frame].origin.x, [diaryWindow frame].origin.y, standardSize.width, standardSize.height);
		[diaryWindow setFrame:newWindowFrame display:YES];
	}
}

- (IBAction)setDefaultDiaryWindowHeight:(id)sender { // ADDED IN RECIPE 10
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	NSSize standardSize = NSSizeFromString([defaults stringForKey:VRDefaultDiaryWindowStandardSizeKey]);
	standardSize.height = (CGFloat)[sender integerValue];
	[defaults setObject:NSStringFromSize(standardSize) forKey:VRDefaultDiaryWindowStandardSizeKey];
	
	// Synchronize text field and stepper.
	if ([sender isKindOfClass:[NSStepper class]]) {
		[[self diaryWindowHeightTextField] takeIntegerValueFrom:sender];
	} else {
		[[self diaryWindowHeightStepper] takeIntegerValueFrom:sender];
	}
	
	// Update window if it is currently open and zoomed to old standard size.
	if ([[sender window] isZoomed]) {
		NSWindow *diaryWindow = [[[[[VRDocumentController sharedDocumentController] diaryDocument] windowControllers] objectAtIndex:0] window];
		NSRect newWindowFrame = NSMakeRect([diaryWindow frame].origin.x, [diaryWindow frame].origin.y + ([diaryWindow frame].size.height - standardSize.height), standardSize.width, standardSize.height);
		[diaryWindow setFrame:newWindowFrame display:YES];
	}
}

- (IBAction)setDefaultDiaryWindowSizeFromCurrent:(id)sender { // ADDED IN RECIPE 10
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	NSWindow *diaryWindow = [[[[[VRDocumentController sharedDocumentController] diaryDocument] windowControllers] objectAtIndex:0] window];
	NSRect diaryWindowRect = [diaryWindow frame];
	[defaults setObject:NSStringFromSize(diaryWindowRect.size) forKey:VRDefaultDiaryWindowStandardSizeKey];
	
	// Synchronize text fields and steppers.
	[[self diaryWindowWidthTextField] setIntegerValue:diaryWindowRect.size.width];
	[[self diaryWindowHeightTextField] setIntegerValue:diaryWindowRect.size.height];
	[[self diaryWindowWidthStepper] setIntegerValue:diaryWindowRect.size.width];
	[[self diaryWindowHeightStepper] setIntegerValue:diaryWindowRect.size.height];
}

- (IBAction)setDefaultDiaryPrintTags:(id)sender { // ADDED IN RECIPE 10
	[[NSUserDefaults standardUserDefaults] setBool:[sender state] forKey:VRDefaultDiaryPrintTagsKey];
}

- (IBAction)setDefaultDiaryPrintHeadersAndFooters:(id)sender { // ADDED IN RECIPE 10
	[[NSUserDefaults standardUserDefaults] setBool:[sender state] forKey:VRDefaultDiaryPrintHeadersAndFootersKey];
}

- (IBAction)setDefaultDiaryPrintTimestamp:(id)sender { // ADDED IN RECIPE 10
	// 0 = Date printed; 1 = Date saved.
	[[NSUserDefaults standardUserDefaults] setInteger:[sender selectedRow] forKey:VRDefaultDiaryPrintTimestampKey];
}

- (IBAction)setDefaultDiaryAutosaveInterval:(id)sender { // ADDED IN RECIPE 10
	NSTimeInterval interval;
	switch ([sender indexOfSelectedItem]) {
		case 0:
			interval = 15.0;
			break;
		case 1:
			interval = 30.0;
			break;
		case 2:
			interval = 60.0;
			break;
		case 3:
			interval = 300.0;
			break;
		case 4:
			interval = 0.0;
			break;
	}
	[[NSUserDefaults standardUserDefaults] setDouble:interval forKey:VRDefaultDiaryAutosaveIntervalKey];
}

- (IBAction)setDefaultCurrentDiaryDocument:(id)sender { // ADDED IN RECIPE 10
	NSString *path = [sender stringValue];
	BOOL isDirectory;
	BOOL fileExists = [[NSFileManager defaultManager] fileExistsAtPath:path isDirectory:&isDirectory];
	if (fileExists & !isDirectory) {
		NSString *type;
		BOOL success = [[NSWorkspace sharedWorkspace] getInfoForFile:path application:NULL type:&type];
		if (success && ([type isEqualToString:@"vrdiary"] || [type isEqualToString:@"rtf"])) {
			[[VRDocumentController sharedDocumentController] setCurrentDiaryURL:[NSURL fileURLWithPath:path]];
		} else {
			[[self alertWrongFileType] beginSheetModalForWindow:[self window] modalDelegate:nil didEndSelector:NULL contextInfo:NULL];
		}
	} else {
		[[self alertNoSuchFile] beginSheetModalForWindow:[self window] modalDelegate:nil didEndSelector:NULL contextInfo:NULL];
	}
}

#pragma mark OVERRIDE METHODS

- (void)windowDidLoad { // ADDED IN RECIPE 10	
	// Set window features.
    NSWindow *window = [self window];
	[window center];
    [window setExcludedFromWindowsMenu:YES];
	
	NSTabViewItem *selectedTabViewItem = [(NSTabView *)[window initialFirstResponder] selectedTabViewItem]; // the tab view is set as the window's initial first responder in Interface Builder
	[window setTitle:[selectedTabViewItem label]];
	
	// Register notification observer for external changes to persistent user defaults.
	NSNotificationCenter *defaultCenter = [NSNotificationCenter defaultCenter];
#if MAC_OS_X_VERSION_MIN_REQUIRED < MAC_OS_X_VERSION_10_6
	[defaultCenter addObserver:self selector:@selector(userDefaultsDidChange:) name:NSUserDefaultsDidChangeNotification object:nil];
#else
	[self setUserDefaultsDidChangeObserver:[defaultCenter addObserverForName:NSUserDefaultsDidChangeNotification object:nil queue:nil usingBlock:^(NSNotification *notification) {
		[self userDefaultsDidChange:notification];
	}]];
#endif
	
	// Set up user interface elements.
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	
	// Recipes pane.
	NSSize recipesWindowSize = NSSizeFromString([defaults stringForKey:VRDefaultRecipesWindowStandardSizeKey]);
	[[self recipesWindowWidthTextField] setIntegerValue:(NSInteger)recipesWindowSize.width];
	[[self recipesWindowHeightTextField] setIntegerValue:(NSInteger)recipesWindowSize.height];
	[[self recipesWindowWidthStepper] setIntegerValue:(NSInteger)recipesWindowSize.width];
	[[self recipesWindowHeightStepper] setIntegerValue:(NSInteger)recipesWindowSize.height];
	
	// Chef's Diary pane.
	NSSize diaryWindowSize = NSSizeFromString([defaults stringForKey:VRDefaultDiaryWindowStandardSizeKey]);
	[[self diaryWindowWidthTextField] setIntegerValue:(NSInteger)diaryWindowSize.width];
	[[self diaryWindowHeightTextField] setIntegerValue:(NSInteger)diaryWindowSize.height];
	[[self diaryWindowWidthStepper] setIntegerValue:(NSInteger)diaryWindowSize.width];
	[[self diaryWindowHeightStepper] setIntegerValue:(NSInteger)diaryWindowSize.height];
	
	double interval = [defaults doubleForKey:VRDefaultDiaryAutosaveIntervalKey];
	NSInteger idx;
	if (interval == 15.0) {
		idx = 0;
	} else if (interval == 30.0) {
		idx = 1;
	} else if (interval == 60.0) {
		idx = 2;
	} else if (interval == 300.0) {
		idx = 3;
	} else if (interval == 0.0) {
		idx = 4;
	}
	[[self diaryAutosaveIntervalButton] selectItemAtIndex:idx];
}

#pragma mark DELEGATE METHODS

- (void)tabView:(NSTabView *)tabView willSelectTabViewItem:(NSTabViewItem *)tabViewItem { // ADDED IN RECIPE 10
	// Delegate method per NSTabView.
	[[self window] setTitle:[tabViewItem label]];
}

- (void)windowDidBecomeKey:(NSNotification *)notification { // ADDED IN RECIPE 10
	// Delegate method per NSWindow.
	// Set up user interface elements.
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	
	// General pane, controls that can be changed externally.
	[[self suppressScaleUpAlertCheckbox] setState:[defaults boolForKey:VRDefaultSuppressScaleUpAlertKey]];
	[[self suppressRestoreAutosavedDocumentAlertCheckbox] setState:[defaults boolForKey:VRDefaultSuppressRestoreDiaryDocumentAlertKey]];	
	
	// Chef's Diary pane, controls that can be changed externally.
	[[self printTagsCheckbox] setState:([defaults boolForKey:VRDefaultDiaryPrintTagsKey]) ? NSOnState : NSOffState];
	[[self printHeadersAndFootersCheckbox] setState:([defaults boolForKey:VRDefaultDiaryPrintHeadersAndFootersKey]) ? NSOnState : NSOffState];
	[[self printTimestampRadioGroup] setState:NSOnState atRow:[defaults integerForKey:VRDefaultDiaryPrintTimestampKey] column:0];
	
	NSString *path = [[[VRDocumentController sharedDocumentController] currentDiaryURL] path];
	if (path) [[self currentDiaryDocumentTextField] setStringValue:path];
}

- (void)windowDidUpdate:(NSNotification *)notification { // ADDED IN RECIPE 10
	// Delegate method per NSWindow.
	[self updateWindow];
}

#pragma mark NOTIFICATION METHODS

- (void)userDefaultsDidChange:(NSNotification *)notification { // ADDED IN RECIPE 10
	// Notification method triggered by NSUserDefaultsDidChangeNotification.
	NSUserDefaults *defaults = [notification object];
	
	// General pane, controls that can be changed externally.
	[[self suppressScaleUpAlertCheckbox] setState:[defaults boolForKey:VRDefaultSuppressScaleUpAlertKey]];
	[[self suppressRestoreAutosavedDocumentAlertCheckbox] setState:[defaults boolForKey:VRDefaultSuppressRestoreDiaryDocumentAlertKey]];
	
	// Chef's Diary pane, controls that can be changed externally.
	[[self printTagsCheckbox] setState:([defaults boolForKey:VRDefaultDiaryPrintTagsKey]) ? NSOnState : NSOffState];
	[[self printHeadersAndFootersCheckbox] setState:([defaults boolForKey:VRDefaultDiaryPrintHeadersAndFootersKey]) ? NSOnState : NSOffState];
	[[self printTimestampRadioGroup] setState:NSOnState atRow:[defaults integerForKey:VRDefaultDiaryPrintTimestampKey] column:0];
	
	NSString *path = [[[VRDocumentController sharedDocumentController] currentDiaryURL] path];
	if (path) [[self currentDiaryDocumentTextField] setStringValue:path];
}

#pragma mark USER INTERFACE VALIDATION

- (void)updateWindow { // ADDED IN RECIPE 10
	// Called by -windowDidUpdate: delegate method.
	
	// Validate (enable or disable) validated controls.
	for (id thisTabViewItem in [(NSTabView *)[[self window] initialFirstResponder] tabViewItems]) {
		for (id thisView in [[thisTabViewItem view] subviews]) {
			if ([thisView conformsToProtocol:@protocol(VRValidatedControl)]) {
				[thisView validate];
			}
		}
	}
}

- (BOOL)validateUserInterfaceItem:(id <NSValidatedUserInterfaceItem>)item { // ADDED IN RECIPE 10
	// Protocol method per NSUserInterfaceValidations protocol. Called by -updateWindow and automatically by -[NSMenu validateMenuItem:].
	SEL action = [item action];
	if (action == @selector(setDefaultRecipesWindowSizeFromCurrent:)) {
		NSWindow *recipesWindow = [[[[[VRDocumentController sharedDocumentController] recipesDocument] windowControllers] objectAtIndex:0] window];
		return (recipesWindow != nil);
	} else if (action == @selector(setDefaultDiaryWindowSizeFromCurrent:)) {
		NSWindow *diaryWindow = [[[[[VRDocumentController sharedDocumentController] diaryDocument] windowControllers] objectAtIndex:0] window];
		return (diaryWindow != nil);
	}
	return YES;
}

#pragma mark ALERTS

- (NSAlert *)alertNoSuchFile { // ADDED IN RECIPE 10
	NSBeep();
	NSAlert *alert = [[[NSAlert alloc] init] autorelease];
	[alert setMessageText:NSLocalizedString(@"The file was not found.", @"message text for NoSuchFile alert")];
	[alert setInformativeText:[NSString stringWithFormat:NSLocalizedString(@"No file exists at %@. Check the path and try again.", @"informative text for NoSuchFile alert"), [[self currentDiaryDocumentTextField] stringValue]]];
	return alert;
}

- (NSAlert *)alertWrongFileType { // ADDED IN RECIPE 10
	NSBeep();
	NSAlert *alert = [[[NSAlert alloc] init] autorelease];
	[alert setMessageText:NSLocalizedString(@"The file is the wrong type.", @"message text for alertWrongFileType alert")];
	[alert setInformativeText:[NSString stringWithFormat:NSLocalizedString(@"The file at %@ is not a Vermont Recipes Chef's Diary file or a Rich Text Format file. Enter a path ending in “.vrdiary” or “.rtf” and try again.", @"informative text for alertWrongFileType alert"), [[self currentDiaryDocumentTextField] stringValue]]];
	return alert;
}

@end


#pragma mark -

@implementation ValidatedPreferencesButton // ADDED IN RECIPE 10

- (void)validate {
	id validator = [NSApp targetForAction:[self action] to:[self target] from:self];
	if ((validator == nil) || ![validator respondsToSelector:[self action]]) {
		[self setEnabled:NO];
	} else if ([validator respondsToSelector:@selector(validateControl:)]) {
		[self setEnabled:[validator validateControl:self]];
	} else if ([validator respondsToSelector:@selector(validateUserInterfaceItem:)]) {
		[self setEnabled:[validator validateUserInterfaceItem:self]];
	} else {
		[self setEnabled:YES];
	}
}

@end
