//
//  CreateCommand.m
//  Vermont Recipes 2.0.0
//
//  Created by Bill Cheeseman on 2010-01-05.
//  Copyright 2000-2010 Bill Cheeseman. All rights reserved.
//
//  CREATED IN RECIPE 12

#import "CreateCommand.h"
#import "AppleScriptLog.h" // ADDED IN RECIPE 12
#import "DiaryDocument.h" // ADDED IN RECIPE 12
#import "DiaryDocument+VRAppleScriptAdditions.h" // ADDED IN RECIPE 12
#import "DiaryEntry.h" // ADDED IN RECIPE 12


@implementation CreateCommand

- (id)performDefaultImplementation { // ADDED IN RECIPE 12
	FourCharCode classCode = [[self createClassDescription] appleEventCode];
	if (classCode == 'dEnt') {

		// Get the container object, a persistent document.
		// NSCreateCommand does not have a direct parameter or receivers, so black magic must be used to get the container specifier.
		NSAppleEventDescriptor *containerDescriptor = [[self appleEvent] attributeDescriptorForKeyword:'subj']; // keySubjectAttr, in Carbon.framework/OpenScripting.framework/ASRegistry.h
		NSScriptObjectSpecifier *containerSpecifier = [NSScriptObjectSpecifier objectSpecifierWithDescriptor:containerDescriptor];
		
		if ([[containerSpecifier keyClassDescription] appleEventCode] == 'dDoc') {
			DiaryDocument *diaryDocument = [containerSpecifier objectsByEvaluatingSpecifier];
			if (diaryDocument != nil) {
				
				// Add current date to properties if script did not include 'date' property.
				NSDictionary *resolvedKeyDictionary = [self resolvedKeyDictionary];
				if ([resolvedKeyDictionary objectForKey:@"date"] == nil) {
					NSMutableDictionary *tempDictionary = [NSMutableDictionary dictionaryWithObject:[NSDate date] forKey:@"date"];
					[tempDictionary addEntriesFromDictionary:resolvedKeyDictionary];
					resolvedKeyDictionary = [[tempDictionary copy] autorelease];
				}
				
				// Create new diary entry and insert it in diary document's ordered entries.
				DiaryEntry *diaryEntry = [diaryDocument newScriptingObjectOfClass:[DiaryEntry class] forValueForKey:@"diary entry" withContentsValue:nil properties:resolvedKeyDictionary];
				[diaryDocument insertObject:diaryEntry inOrderedEntriesAtIndex:0]; // index is ignored
				NSScriptObjectSpecifier *containerRef = [diaryDocument objectSpecifier];
				NSScriptObjectSpecifier *specifier = [[NSNameSpecifier alloc] initWithContainerClassDescription:[containerRef keyClassDescription] containerSpecifier:containerRef key:@"orderedEntries" name:[diaryDocument entryTitleForDate:[resolvedKeyDictionary objectForKey:@"date"]]];
				SLOG(@"CreateCommand performDefaultImplementation, returns specifier: %@", specifier);
				return specifier;
			}
		}
	}
	return [super performDefaultImplementation];
}

@end
