//
//  DiaryDocument+VRAppleScriptAdditions.m
//  Vermont Recipes 2.0.0
//
//  Created by Bill Cheeseman on 2010-01-05.
//  Copyright 2000-2010 Bill Cheeseman. All rights reserved.
//
//  CREATED IN RECIPE 12

#import "DiaryDocument+VRAppleScriptAdditions.h"
#import "AppleScriptLog.h" // ADDED IN RECIPE 12
#import "DiaryWindowController.h" // ADDED IN RECIPE 12
#import "DiaryEntry.h" // ADDED IN RECIPE 12


static id tempDiaryEntryText = nil;

@implementation DiaryDocument (VRAppleScriptAdditions) // MODIFIED IN RECIPE 12

#pragma mark ACCESSOR METHODS

- (void)setOrderedEntries:(NSMutableArray *)array { // ADDED IN RECIPE 12
	if (orderedEntries != array) {
		[orderedEntries release];
		orderedEntries = [array retain];
	}
}

- (NSMutableArray *)orderedEntries { // ADDED IN RECIPE 12
	// For AppleScript 'get diary entries of <diary document>'.
	SLOG(@"orderedEntries: %@", orderedEntries);
	if (!orderedEntries) [self setOrderedEntries:[self makeOrderedEntries]];
	return [[orderedEntries retain] autorelease];
}

- (DiaryEntry *)currentDiaryEntry { // ADDED IN RECIPE 12
	// For AppleScript property 'current diary entry of <diary document>'
	DiaryWindowController *windowController = [[self windowControllers] objectAtIndex:0];
	NSUInteger insertionPointIndex = [windowController insertionPointIndex];
	NSUInteger currentEntryLocation = [self currentEntryTitleRangeForIndex:insertionPointIndex].location;
	if (currentEntryLocation == NSNotFound) return (DiaryEntry *)[NSNull null];
	NSRange testEntryRange = [self firstEntryTitleRange];
	NSUInteger idx = 0;
	while (testEntryRange.location != currentEntryLocation) {
		testEntryRange = [self nextEntryTitleRangeForIndex:testEntryRange.location + testEntryRange.length];
		if (testEntryRange.location == NSNotFound) return (DiaryEntry *)[NSNull null];
		idx += 1;
	}
	DiaryEntry *entry = [[self orderedEntries] objectAtIndex:idx];
	SLOG(@"currentDiaryEntry: %@", entry);
	return entry;
}

- (void)setDocumentText:(id)text { // ADDED IN RECIPE 12
	// For AppleScript 'set document text of <diary document> to <text or rich text>'.
	SLOG(@"setDocumentText: %@, of class %@", text, NSStringFromClass([text class]));
	NSRange replacementRange = NSMakeRange(0, [[self diaryDocTextStorage] length]);
	NSString *actionName = NSLocalizedString(@"Set Diary Document Text", @"name of undo action for AppleScript set document text");
	[self updateDocumentInRange:replacementRange withText:text undoActionName:actionName];
}

- (NSTextStorage *)documentText { // ADDED IN RECIPE 12
	// For AppleScript 'get document text of <diary document>'.
	NSTextStorage *storage = [[NSTextStorage alloc] init];
	[storage setAttributedString:[self diaryDocTextStorage]];
	SLOG(@"documentText: %@", storage);
	return [storage autorelease];
}

#pragma mark APPLESCRIPT SUPPORT

- (NSScriptObjectSpecifier *)objectSpecifier { // ADDED IN RECIPE 12
	NSScriptObjectSpecifier *specifier = [[NSNameSpecifier alloc] initWithContainerClassDescription:[NSScriptClassDescription classDescriptionForClass:[NSApp class]] containerSpecifier:nil key:@"orderedDocuments" name:[self lastComponentOfFileName]];
	SLOG(@"Diary Document objectSpecifier: %@", specifier);
	return [specifier autorelease];
}

- (NSMutableArray *)makeOrderedEntries { // ADDED IN RECIPE 12
	// Creates and populates the orderedEntries array to reflect the ranges of all current diary entries. Called by the -orderedEntries accessor method when a script is run after the diary document is first opened or has been edited, whether manually or by AppleScript.
	NSMutableArray *array = [NSMutableArray array];
	NSRange thisEntryRange = [self firstEntryTitleRange];
	if (thisEntryRange.location != NSNotFound) {
		DiaryEntry *entry;
		NSRange nextEntryRange = [self nextEntryTitleRangeForIndex:thisEntryRange.location + thisEntryRange.length];
		while (nextEntryRange.location != NSNotFound) {
			thisEntryRange.length = nextEntryRange.location - thisEntryRange.location;
			entry = [[DiaryEntry alloc] initWithRange:thisEntryRange forDocument:self];
			[array addObject:entry];
			[entry release];
			thisEntryRange = nextEntryRange;
			nextEntryRange = [self nextEntryTitleRangeForIndex:thisEntryRange.location + thisEntryRange.length];
		}
		thisEntryRange.length = [[self diaryDocTextStorage] length] - thisEntryRange.location;
		entry = [[DiaryEntry alloc] initWithRange:thisEntryRange forDocument:self];
		[array addObject:entry];
		[entry release];
	}
	return [array autorelease];
}

- (void)invalidateOrderedEntries { // ADDED IN RECIPE 12
	[self setOrderedEntries:nil];
}

- (void)updateDocumentInRange:(NSRange)range withText:(id)text undoActionName:(NSString *)actionName { // ADDED IN RECIPE 12
	// Updates the diary document's text storage to reflect edits made by AppleScript 'set' commands.
	NSTextView *keyView = [[[self windowControllers] objectAtIndex:0] keyDiaryView];
	NSTextStorage *storage = [self diaryDocTextStorage];
	if ([keyView shouldChangeTextInRange:range replacementString:text]) {
		if ([text length] == 0) {
			[storage deleteCharactersInRange:range];
		} else {
			if ([text isKindOfClass:[NSAttributedString class]]) {
				[storage replaceCharactersInRange:range withAttributedString:text];
			} else {
				[storage replaceCharactersInRange:range withString:text];
			}
		}
		[keyView didChangeText];
		[[keyView undoManager] setActionName:actionName];
	}
}

- (id)newScriptingObjectOfClass:(Class)class forValueForKey:(NSString *)key withContentsValue:(id)contentsValue properties:(NSDictionary *)properties { // ADDED IN RECIPE 12
	// Creates a new diary entry object in response to the AppleScript 'make' command.
	if (class == [DiaryEntry class]) {
		id fullText;
				
		// Get 'date' property. Our CreateCommand subclass of NSCreateCommand always supplies a 'date' property, either provided by the user in a 'with properties' clause, or as the current date. Any 'name' property is ignored.
		NSDate *date = [properties objectForKey:@"date"];
		NSString *titleInsert = [self entryTitleInsertForDate:date];
		
		// Get 'entryText' property, if any.
		id entryText = [properties objectForKey:@"entryText"];
		
		// Combine the 'date' and 'entry text' properties into fullText, ignoring any 'tags' or 'body text' properties.
		if (entryText) {
			if ([entryText isKindOfClass:[NSAttributedString class]]) {
				fullText = [[[NSMutableAttributedString alloc] initWithString:titleInsert] autorelease];
				[fullText appendAttributedString:entryText];
			} else {
				fullText = [NSString stringWithFormat:@"%@%@", titleInsert, entryText];
			}						
		} else {
						
			// Get 'tags' property, if any.
			NSArray *tagsArray = [properties objectForKey:@"tags"];
			NSString *tags;
			if (tagsArray) {
				tags = [[self tagTitleInsert] stringByAppendingString:[tagsArray componentsJoinedByString:@", "]];
				tags = [tags substringFromIndex:1]; // remove leading newline
				tags = [tags stringByAppendingString:@"\n"]; // add trailing newline
			} else {
				tags = @"";
			}
				
			
			// Get 'body text' property, if any.
			id bodyText = [properties objectForKey:@"bodyText"];
			if (!bodyText) bodyText = @"";
			
			// Combine the 'date', 'tags' and 'entry text' properties into fullText.
			if ([bodyText isKindOfClass:[NSAttributedString class]]) {
				fullText = [[NSMutableAttributedString alloc] initWithString:titleInsert];
				if (tags) [fullText appendAttributedString:[[[NSAttributedString alloc] initWithString:tags] autorelease]];
				if (bodyText) [fullText appendAttributedString:bodyText];
			} else {
				fullText = [NSString stringWithFormat:@"%@%@%@", titleInsert, tags, bodyText];
			}
		}
		
		// Set external static variable tempDiaryEntryText for use in -insertInOrderedEntries:atIndex: or -insertObject:inOrderedEntriesAtIndex:.
		tempDiaryEntryText = fullText; // reset to nil in -insertInOrderedEntries:atIndex: or -insertObject:inOrderedEntriesAtIndex:
		
		// Create the diary entry.
		NSRange entryRange = NSMakeRange([[self diaryDocTextStorage] length], [fullText length]);
		DiaryEntry *entry = [[DiaryEntry alloc] initWithRange:entryRange forDocument:self];
		
		SLOG(@"newScriptingObjectOfClass: %@ forValueForKey: %@ withContentsValue: %@ properties: %@", NSStringFromClass(class), key, contentsValue, properties);
		return entry;
	}
	return [super newScriptingObjectOfClass:class forValueForKey:key withContentsValue:contentsValue properties:properties];
}

/* These NSScriptKeyValueCoding informal protocol methods are shown here only for illustration:
- (void)insertInOrderedEntries:(id)entry { // ADDED IN RECIPE 12
	// For AppleScript 'make new diary entry in <diary document>'.
	SLOG(@"insertInOrderedEntries: %@", entry);
	[[self orderedEntries] insertObject:entry atIndex:[[self orderedEntries] count]];
	
	NSRange entryRange = NSMakeRange([[self diaryDocTextStorage] length], 0);
	NSString *actionName = NSLocalizedString(@"Insert Entry", @"name of undo action for AppleScript make new diary entry");
	[self updateDocumentInRange:entryRange withText:tempDiaryEntryText undoActionName:actionName];
	
	tempDiaryEntryText = nil; // set in -newScriptingObjectOfClass:forValueForKey:withContentsValue:properties:
}

- (void)insertInOrderedEntries:(id)entry atIndex:(NSUInteger)index { // ADDED IN RECIPE 12
	// For AppleScript 'make new diary entry at <location> of <diary document> with properties <properties>'.
	SLOG(@"insertInOrderedEntries: %@, atIndex: %lu -- IGNORING INDEX", entry, (unsigned long)index);
	// [[self orderedEntries] insertObject:entry atIndex:index]; // not supported by Vermont Recipes
	[[self orderedEntries] insertObject:entry atIndex:[[self orderedEntries] count]]; // ignore index
	
	NSRange entryRange = NSMakeRange([[self diaryDocTextStorage] length], 0);
	NSString *actionName = NSLocalizedString(@"Insert Entry at Location", @"name of undo action for AppleScript make new diary entry at location");
	[self updateDocumentInRange:entryRange withText:tempDiaryEntryText undoActionName:actionName];
	
	tempDiaryEntryText = nil; // set in -newScriptingObjectOfClass:forValueForKey:withContentsValue:properties:
}

- (void)removeFromOrderedEntriesAtIndex:(NSUInteger)index { // ADDED IN RECIPE 12
	// For AppleScript 'delete diary entry <index> of <diary document>'.
	SLOG(@"removeFromOrderedEntries: index: %lu", (unsigned long)index);
	NSRange entryRange = [[[self orderedEntries] objectAtIndex:index] entryRange];
	[[self orderedEntries] removeObjectAtIndex:index];
	
	NSString *actionName = NSLocalizedString(@"Delete Entry", @"name of undo action for AppleScript delete diary entry");
	[self updateDocumentInRange:entryRange withText:@"" undoActionName:actionName];
}
*/

#pragma mark KVC METHODS SUPPORTING APPLESCRIPT ELEMENTS
#pragma mark orderedEntries

- (void)insertObject:(id)entry inOrderedEntriesAtIndex:(NSUInteger)index { // ADDED IN RECIPE 12
	// For AppleScript 'make new diary entry of <diary document>'.
	SLOG(@"insertObject: %@, inOrderedEntriesAtIndex: %lu -- IGNORING INDEX", entry, (unsigned long)index);
	[[self orderedEntries] insertObject:entry atIndex:[[self orderedEntries] count]]; // ignore index
	
	NSRange entryRange = NSMakeRange([[self diaryDocTextStorage] length], 0);
	NSString *actionName = NSLocalizedString(@"Insert Entry", @"name of undo action for AppleScript make new diary entry");
	[self updateDocumentInRange:entryRange withText:tempDiaryEntryText undoActionName:actionName];
	
	tempDiaryEntryText = nil; // set in -newScriptingObjectOfClass:forValueForKey:withContentsValue:properties:
}

- (void)removeObjectFromOrderedEntriesAtIndex:(NSUInteger)index { // ADDED IN RECIPE 12
	// For AppleScript 'delete diary entry <index> of <diary document>'.
	SLOG(@"removeObjectFromOrderedEntries: index: %lu", (unsigned long)index);
	NSRange entryRange = [[[self orderedEntries] objectAtIndex:index] entryRange];
	[[self orderedEntries] removeObjectAtIndex:index];
	
	NSString *actionName = NSLocalizedString(@"Delete Entry", @"name of undo action for AppleScript delete diary entry");
	[self updateDocumentInRange:entryRange withText:@"" undoActionName:actionName];
}

#pragma mark DELEGATE METHODS

- (void)textStorageDidProcessEditing:(NSNotification *)notification { // ADDED IN RECIPE 12
	// Delegate method per NSTextStorage.
	[self invalidateOrderedEntries]; // ADDED IN RECIPE 12
}

@end
