//
//  DiaryEntry.m
//  Vermont Recipes 2.0.0
//
//  Copyright 2000-2010 Bill Cheeseman. All rights reserved.
//
//  CREATED IN RECIPE 12

#import "DiaryEntry.h"
#import "AppleScriptLog.h" // ADDED IN RECIPE 12
#import "DiaryDocument.h" // ADDED IN RECIPE 12
#import "DiaryDocument+VRAppleScriptAdditions.h" // ADDED IN RECIPE 12
#import "DiaryWindowController.h" // ADDED IN RECIPE 12
#import "NSMutableString+VRMutableStringAdditions.h" // ADDED IN RECIPE 12


@implementation DiaryEntry

#pragma mark INITIALIZATION

- (id)init { // ADDED IN RECIPE 12
	return [self initWithRange:NSMakeRange(0, 0) forDocument:nil];
}

- (id)initWithRange:(NSRange)range forDocument:(DiaryDocument *)document { // ADDED IN RECIPE 12
	// Designated initializer for updating existing ordered entries.
	if ((self = [super init])) {
		entryRange = range;
		diaryDocument = [document retain];
	}
	return self;
}

- (void)dealloc {
	[diaryDocument release];
	[super dealloc];
}

#pragma mark ACCESSOR METHODS

- (NSRange)entryRange { // ADDED IN RECIPE 12
	return entryRange;
}

- (DiaryDocument *)diaryDocument { // ADDED IN RECIPE 12
	return diaryDocument;
}

- (void)setName:(NSString *)name { // ADDED IN RECIPE 12
	// For Applescript 'set name of <diary entry> to <name>'.
	SLOG(@"setname: %@", name);
	NSString *nameString = [@" " stringByAppendingString:name]; // prepend single space
	NSRange nameRange = [[self diaryDocument] rangeOfLineFromMarkerIndex:[self entryRange].location];
	nameRange.location += 1;
	nameRange.length -= 1;
	NSRange nameRangeInDocument = NSMakeRange([self entryRange].location + nameRange.location, nameRange.length);
	
	NSString *actionName = NSLocalizedString(@"Set Diary Entry Name", @"name of undo action for AppleScript set name");
	[[self diaryDocument] updateDocumentInRange:nameRangeInDocument withText:nameString undoActionName:actionName];
}

- (NSString *)name { // ADDED IN RECIPE 12
	// For AppleScript 'get name of <diary entry>'.
	NSString *entryString = [[[[self diaryDocument] diaryDocTextStorage] string] substringWithRange:[self entryRange]];
	
	NSString *nameString;
	NSRange nameRange = [DiaryDocument rangeOfLineFromMarkerIndex:0 inString:entryString];
	if (nameRange.length <= 2) {
		nameString = (NSString *)[NSNull null];
	} else {
		nameRange.location += 2;
		nameRange.length -= 2;
		nameString = [entryString substringWithRange:nameRange];
		if (nameString == nil) nameString = (NSString *)[NSNull null];		
	}
	SLOG(@"name: %@", nameString);
	return nameString;
}

- (void)setDate:(NSDate *)date { // ADDED IN RECIPE 12
	// For Applescript 'set date of <diary entry> to <date>'.
	SLOG(@"setdate: %@", date);
	NSString *dateString = [[self diaryDocument] entryTitleForDate:date];
	dateString = [@" " stringByAppendingString:dateString]; // prepend single space
	NSRange dateRange = [[self diaryDocument] rangeOfLineFromMarkerIndex:[self entryRange].location];
	dateRange.location += 1;
	dateRange.length -= 1;
	NSRange dateRangeInDocument = NSMakeRange([self entryRange].location + dateRange.location, dateRange.length);
	
	NSString *actionName = NSLocalizedString(@"Set Diary Entry Date", @"name of undo action for AppleScript set date");
	[[self diaryDocument] updateDocumentInRange:dateRangeInDocument withText:dateString undoActionName:actionName];
}

- (NSDate *)date { // ADDED IN RECIPE 12
	// For AppleScript 'get date of <diary entry>'.
	NSDate *date;
	NSRange dateRange = [[self diaryDocument] rangeOfLineFromMarkerIndex:[self entryRange].location];
	if (dateRange.length <= 2) {
		date = (NSDate *)[NSNull null];
	} else {
		date = [[self diaryDocument] dateFromEntryTitleRange:dateRange];
		if (date == nil) date = (NSDate *)[NSNull null];
	}
	SLOG(@"date: %@", date);
	return date;
}

- (void)setTags:(NSArray *)tags { // ADDED IN RECIPE 12
	// For AppleScript 'set tags of <diary entry> to <array of text>'.
	NSString *tagsString = [[[self diaryDocument] tagTitleInsert] stringByAppendingString:[tags componentsJoinedByString:@", "]];
	SLOG(@"setTags: %@", tagsString);

	NSString *entryString = [[[[self diaryDocument] diaryDocTextStorage] string] substringWithRange:[self entryRange]];
	NSRange tagsRange = [DiaryDocument firstTagRangeInString:entryString];
	if (tagsRange.location == NSNotFound) {
		tagsRange.location = [DiaryDocument rangeOfLineFromMarkerIndex:0 inString:entryString].length;
	}
	tagsRange.location += [self entryRange].location - 1;
	tagsRange.length += 1;
	
	NSString *actionName = NSLocalizedString(@"Set Diary Entry Tags", @"name of undo action for AppleScript set tags");
	[[self diaryDocument] updateDocumentInRange:tagsRange withText:tagsString undoActionName:actionName];
}

- (NSArray *)tags { // ADDED IN RECIPE 12
	// For AppleScript 'get tags of <diary entry>'.
	NSString *entryString = [[[[self diaryDocument] diaryDocTextStorage] string] substringWithRange:[self entryRange]];
	NSRange tagsRange = [DiaryDocument firstTagRangeInString:entryString];
	if (tagsRange.location == NSNotFound) return [NSArray array];
	NSString *tagsString = [entryString substringWithRange:tagsRange];
	NSString *prefixString = [[[self diaryDocument] tagTitle] stringByAppendingString:@": "];
	NSRange prefixRange = [tagsString rangeOfString:prefixString];
	if (prefixRange.location == NSNotFound) return [NSArray array];
	tagsString = [tagsString substringFromIndex:prefixRange.location + prefixRange.length];
	NSArray *tagsArray = [tagsString componentsSeparatedByCharactersInSet:[NSCharacterSet punctuationCharacterSet]];
	NSMutableArray *returnArray = [NSMutableArray array];
	for (NSString *thisTag in tagsArray) {
		thisTag = [thisTag stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];
		[returnArray addObject:thisTag];
	}
	SLOG(@"tags: %@", returnArray);
	return [[returnArray copy] autorelease];
}

- (void)setBodyText:(id)text { // ADDED IN RECIPE 12
	// For AppleScript 'set body text of <diary entry> to <text or rich text>'.
	SLOG(@"setBodyText: %@, of class: %@", text, NSStringFromClass([text class]));
	NSRange bodyRangeInDocument = [self bodyTextRange];
	
	NSString *trailingWhitespaceString = @"\n\n";
	if (bodyRangeInDocument.location + bodyRangeInDocument.length == [[[self diaryDocument] diaryDocTextStorage] length]) {
		trailingWhitespaceString = @"";
	}
	
	if ([text isKindOfClass:[NSAttributedString class]]) {
		[text appendAttributedString:[[[NSAttributedString alloc] initWithString:trailingWhitespaceString] autorelease]];
	} else {
		text = [text stringByAppendingString:trailingWhitespaceString];
	}
	
	NSString *actionName = NSLocalizedString(@"Set Diary Entry Body Text", @"name of undo action for AppleScript set body text");
	[[self diaryDocument] updateDocumentInRange:bodyRangeInDocument withText:text undoActionName:actionName];
}

- (NSTextStorage *)bodyText { // ADDED IN RECIPE 12
	// For AppleScript 'get body text of <diary entry>'.
	NSRange bodyRangeInDocument = [self bodyTextRange];
	if (bodyRangeInDocument.length == 0) {
		return (NSTextStorage *)[NSNull null];
	} else {
		NSString *bodyString = [[[[self diaryDocument] diaryDocTextStorage] string] substringWithRange:bodyRangeInDocument];
		
		NSString *trimmedBodyString = [bodyString stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]];
		NSRange trimmedBodyRange = [bodyString rangeOfString:trimmedBodyString];
		NSRange trimmedBodyRangeInDocument = NSMakeRange(bodyRangeInDocument.location + trimmedBodyRange.location, trimmedBodyRange.length);
		
		NSAttributedString *text = [[[self diaryDocument] diaryDocTextStorage] attributedSubstringFromRange:trimmedBodyRangeInDocument];
		NSTextStorage *storage = [[NSTextStorage alloc] init];
		[storage setAttributedString:text];
		SLOG(@"bodyText: %@", storage);
		return [storage autorelease];
	}
}

- (void)setEntryText:(id)text { // ADDED IN RECIPE 12
	// For AppleScript 'set entry text of <diary entry> to <text or rich text>'.
	SLOG(@"setEntryText: %@, of class %@", text, NSStringFromClass([text class]));
	NSString *trailingWhitespaceString = @"\n\n";
	if ([self entryRange].location + [self entryRange].length == [[[self diaryDocument] diaryDocTextStorage] length]) {
		trailingWhitespaceString = @"";
	}
	
	if ([text isKindOfClass:[NSAttributedString class]]) {
		[text appendAttributedString:[[[NSAttributedString alloc] initWithString:trailingWhitespaceString] autorelease]];
	} else {
		text = [text stringByAppendingString:trailingWhitespaceString];
	}
	
	NSString *actionName = NSLocalizedString(@"Set Diary Entry Text", @"name of undo action for AppleScript set entry text");
	[[self diaryDocument] updateDocumentInRange:[self entryRange] withText:text undoActionName:actionName];
}

- (NSTextStorage *)entryText { // ADDED IN RECIPE 12
	// For AppleScript 'get entry text of <diary entry>'.
	NSString *entryString = [[[[self diaryDocument] diaryDocTextStorage] string] substringWithRange:[self entryRange]];

	NSString *trimmedEntryString = [entryString stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]];
	NSRange trimmedEntryRange = [entryString rangeOfString:trimmedEntryString];
	NSRange trimmedEntryRangeInDocument = NSMakeRange([self entryRange].location + trimmedEntryRange.location, trimmedEntryRange.length);
	
	NSAttributedString *text = [[[self diaryDocument] diaryDocTextStorage] attributedSubstringFromRange:trimmedEntryRangeInDocument];
	NSTextStorage *storage = [[NSTextStorage alloc] init];
	[storage setAttributedString:text];
	SLOG(@"entryText: %@", storage);
	return [storage autorelease];
}

#pragma mark APPLESCRIPT SUPPORT

- (NSScriptObjectSpecifier *)objectSpecifier { // ADDED IN RECIPE 12
	NSMutableArray *orderedEntries = [[self diaryDocument] orderedEntries];
	NSUInteger idx = [orderedEntries indexOfObjectIdenticalTo:self];
	if (idx == NSNotFound) return nil;
	NSScriptObjectSpecifier *containerRef = [[self diaryDocument] objectSpecifier];
	NSScriptObjectSpecifier *specifier = [[NSIndexSpecifier alloc] initWithContainerClassDescription:[containerRef keyClassDescription] containerSpecifier:containerRef key:@"orderedEntries" index:idx];
	SLOG(@"Diary Entry objectSpecifier: %@", specifier);
	return [specifier autorelease];
}

- (NSComparisonResult)compareDiaryEntry:(DiaryEntry *)entry { // ADDED IN RECIPE 12
	NSTimeInterval interval = [[self date] timeIntervalSinceDate:[entry date]];
	if (interval < 0) {
		return NSOrderedAscending;
	} else if (interval > 0) {
		return NSOrderedDescending;
	}
	return NSOrderedSame;
}
						

- (NSComparisonResult)reverseCompareDiaryEntry:(DiaryEntry *)entry { // ADDED IN RECIPE 12
	NSTimeInterval interval = [[self date] timeIntervalSinceDate:[entry date]];
	if (interval < 0) {
		return NSOrderedDescending;
	} else if (interval > 0) {
		return NSOrderedAscending;
	}
	return NSOrderedSame;
}

- (id)handleEncryptCommand:(NSScriptCommand *)command { // ADDED IN RECIPE 12
	// Encrypt body text using ROT13 algorithm.
	DiaryEntry *diaryEntry = [command evaluatedReceivers];
	SLOG(@"handleEncryptCommand, diaryEntry: %@", diaryEntry);
	
	// Encrypt body text.
	NSTextStorage *bodyText = [diaryEntry bodyText];
	if (![bodyText isKindOfClass:[NSNull class]]) {
		NSMutableString *bodyString = [bodyText mutableString];
		if (bodyString != nil) {
			[bodyString VR_rotateStringBy13:bodyString];
			
			// Install and display encrypted body text with undo/redo support.
			NSRange bodyRangeInDocument = [self bodyTextRange];
			if (bodyRangeInDocument.location + bodyRangeInDocument.length < [[[diaryEntry diaryDocument] diaryDocTextStorage] length]) {
				[bodyString appendString:@"\n\n"];
			}
			
			NSString *actionName = NSLocalizedString(@"Encrypt Diary Entry", @"name of undo action for AppleScript encrypt");
			[[self diaryDocument] updateDocumentInRange:bodyRangeInDocument withText:bodyString undoActionName:actionName];
		}
	}
	return nil;
}

- (id)handleDecryptCommand:(NSScriptCommand *)command { // ADDED IN RECIPE 12
	// Decrypt body text using ROT13 algorithm.
	DiaryEntry *diaryEntry = [command evaluatedReceivers];
	SLOG(@"handleDecryptCommand: %@", diaryEntry);
	
	// Decrypt body text.
	NSTextStorage *bodyText = [diaryEntry bodyText];
	if (![bodyText isKindOfClass:[NSNull class]]) {
		NSMutableString *bodyString = [bodyText mutableString];
		if (bodyString != nil) {
			[bodyString VR_rotateStringBy13:bodyString];
			
			// Install and display decrypted body text with undo/redo support.
			NSRange bodyRangeInDocument = [self bodyTextRange];
			if (bodyRangeInDocument.location + bodyRangeInDocument.length < [[[diaryEntry diaryDocument] diaryDocTextStorage] length]) {
				[bodyString appendString:@"\n\n"];
			}
			
			NSString *actionName = NSLocalizedString(@"Decrypt Diary Entry", @"name of undo action for AppleScript decrypt");
			[[self diaryDocument] updateDocumentInRange:bodyRangeInDocument withText:bodyString undoActionName:actionName];
		}
	}
	return nil;
}

- (NSRange)bodyTextRange { // ADDED IN RECIPE 12
	// Returns the range of the entry's body text in the document, including any trailing newlines.
	NSString *entryString = [[[[self diaryDocument] diaryDocTextStorage] string] substringWithRange:[self entryRange]];
	NSRange nameRange = [DiaryDocument rangeOfLineFromMarkerIndex:0 inString:entryString];
	NSRange tagsRange = [DiaryDocument firstTagRangeInString:entryString];
	NSUInteger bodyLocation = (tagsRange.location == NSNotFound) ? nameRange.length + 1 : nameRange.length + tagsRange.length + 2;
	return NSMakeRange(bodyLocation + [self entryRange].location, [self entryRange].length - bodyLocation);
}

@end
