//
//  PreferencesWindowController.h
//  Vermont Recipes 2.0.0
//
//  Created by Bill Cheeseman on 2009-11-29.
//  Copyright 2000-2009 Bill Cheeseman. All rights reserved.
//
//  CREATED IN RECIPE 10

#import <Cocoa/Cocoa.h>
#import "ValidationProtocols.h"


@interface PreferencesWindowController : NSWindowController <NSUserInterfaceValidations> {
	id userDefaultsDidChangeObserver; // ADDED IN RECIPE 10

	// General pane.
	IBOutlet NSButton *suppressScaleUpAlertCheckbox; // ADDED IN RECIPE 10
	IBOutlet NSButton *suppressRestoreAutosavedDocumentAlertCheckbox; // ADDED IN RECIPE 10
	
	// Recipes pane.
	IBOutlet NSTextField *recipesWindowWidthTextField; // ADDED IN RECIPE 10
	IBOutlet NSTextField *recipesWindowHeightTextField; // ADDED IN RECIPE 10
	IBOutlet NSStepper *recipesWindowWidthStepper; // ADDED IN RECIPE 10
	IBOutlet NSStepper *recipesWindowHeightStepper; // ADDED IN RECIPE 10
	
	// Chef's Diary pane.
	IBOutlet NSTextField *diaryWindowWidthTextField; // ADDED IN RECIPE 10
	IBOutlet NSTextField *diaryWindowHeightTextField; // ADDED IN RECIPE 10
	IBOutlet NSStepper *diaryWindowWidthStepper; // ADDED IN RECIPE 10
	IBOutlet NSStepper *diaryWindowHeightStepper; // ADDED IN RECIPE 10	

	IBOutlet NSButton *printTagsCheckbox; // ADDED IN RECIPE 10
	IBOutlet NSButton *printHeadersAndFootersCheckbox; // ADDED IN RECIPE 10
	IBOutlet NSMatrix *printTimestampRadioGroup; // ADDED IN RECIPE 10
	
	IBOutlet NSPopUpButton *diaryAutosaveIntervalButton; // ADDED IN RECIPE 10
	
	IBOutlet NSTextField *currentDiaryDocumentTextField; // ADDED IN RECIPE 10
}

#pragma mark FACTORY METHOD

+ (PreferencesWindowController *)sharedController; // ADDED IN RECIPE 10

#pragma mark ACCESSOR METHODS

- (void)setUserDefaultsDidChangeObserver:(id)observer; // ADDED IN RECIPE 10
- (id)userDefaultsDidChangeObserver; // ADDED IN RECIPE 10

// General pane.
- (NSButton *)suppressScaleUpAlertCheckbox; // ADDED IN RECIPE 10
- (NSButton *)suppressRestoreAutosavedDocumentAlertCheckbox; // ADDED IN RECIPE 10

// Recipes pane.
- (NSTextField *)recipesWindowWidthTextField; // ADDED IN RECIPE 10
- (NSTextField *)recipesWindowHeightTextField; // ADDED IN RECIPE 10
- (NSStepper *)recipesWindowWidthStepper; // ADDED IN RECIPE 10
- (NSStepper *)recipesWindowHeightStepper; // ADDED IN RECIPE 10

// Chef's Diary pane.
- (NSTextField *)diaryWindowWidthTextField; // ADDED IN RECIPE 10
- (NSTextField *)diaryWindowHeightTextField; // ADDED IN RECIPE 10
- (NSStepper *)diaryWindowWidthStepper; // ADDED IN RECIPE 10
- (NSStepper *)diaryWindowHeightStepper; // ADDED IN RECIPE 10

- (NSButton *)printTagsCheckbox; // ADDED IN RECIPE 10
- (NSButton *)printHeadersAndFootersCheckbox; // ADDED IN RECIPE 10
- (NSMatrix *)printTimestampRadioGroup; // ADDED IN RECIPE 10

- (NSPopUpButton *)diaryAutosaveIntervalButton ; // ADDED IN RECIPE 10

- (NSTextField *)currentDiaryDocumentTextField; // ADDED IN RECIPE 10

#pragma mark ACTION METHODS

// General pane.
- (IBAction)setDefaultSuppressScaleUpAlert:(id)sender; // ADDED IN RECIPE 10
- (IBAction)setDefaultSuppressRestoreAutosavedDocumentAlert:(id)sender; // ADDED IN RECIPE 10

// Recipes pane.
- (IBAction)setDefaultRecipesWindowWidth:(id)sender; // ADDED IN RECIPE 10
- (IBAction)setDefaultRecipesWindowHeight:(id)sender; // ADDED IN RECIPE 10
- (IBAction)setDefaultRecipesWindowSizeFromCurrent:(id)sender; // ADDED IN RECIPE 10

// Chef's Diary pane.
- (IBAction)setDefaultDiaryWindowWidth:(id)sender; // ADDED IN RECIPE 10
- (IBAction)setDefaultDiaryWindowHeight:(id)sender; // ADDED IN RECIPE 10
- (IBAction)setDefaultDiaryWindowSizeFromCurrent:(id)sender; // ADDED IN RECIPE 10

- (IBAction)setDefaultDiaryPrintTags:(id)sender; // ADDED IN RECIPE 10
- (IBAction)setDefaultDiaryPrintHeadersAndFooters:(id)sender; // ADDED IN RECIPE 10
- (IBAction)setDefaultDiaryPrintTimestamp:(id)sender; // ADDED IN RECIPE 10

- (IBAction)setDefaultDiaryAutosaveInterval:(id)sender; // ADDED IN RECIPE 10

- (IBAction)setDefaultCurrentDiaryDocument:(id)sender; // ADDED IN RECIPE 10

#pragma mark NOTIFICATION METHODS
- (void)userDefaultsDidChange:(NSNotification *)notification; // ADDED IN RECIPE 10

#pragma mark USER INTERFACE VALIDATION

- (void)updateWindow; // ADDED IN RECIPE 10

#pragma mark ALERTS

- (NSAlert *)alertNoSuchFile; // ADDED IN RECIPE 10
- (NSAlert *)alertWrongFileType; // ADDED IN RECIPE 10

@end


#pragma mark -

@interface ValidatedPreferencesButton : NSButton <VRValidatedControl> { // ADDED IN RECIPE 10
}

@end
