//
//  SortCommand.h
//  Vermont Recipes 2.0.0
//
//  Created by Bill Cheeseman on 2010-01-15.
//  Copyright 2000-2009 Bill Cheesseman. All rights reserved.
//
//  CREATED IN RECIPE 12

#import "SortCommand.h"
#import "AppleScriptLog.h" // ADDED IN RECIPE 12
#import "DiaryDocument.h" // ADDED IN RECIPE 12
#import "VRDocumentController.h" // ADDED IN RECIPE 12
#import "DiaryEntry.h" // ADDED IN RECIPE 12
#import "DiaryDocument+VRAppleScriptAdditions.h" // ADDED IN RECIPE 12


@implementation SortCommand

#pragma mark OVERRIDE METHODS

- (id)performDefaultImplementation { // ADDED IN RECIPE 12
	id directParameter = [self directParameter];
	NSDictionary *arguments = [self evaluatedArguments];
	SLOG(@"SortCommand performDefaultImplementation, directParameter: %@, arguments: %@", directParameter, arguments);
	
	// Get diary document and its ordered entries.
	NSDocument *document = [directParameter objectsByEvaluatingSpecifier];
	NSMutableArray *orderedEntries;	
	if ([document isKindOfClass:[DiaryDocument class]]) {
		orderedEntries = [(DiaryDocument *)document orderedEntries];
		if ((orderedEntries != nil) && ([orderedEntries count] > 1)) {
			
			
			// Sort diary document's ordered entries.
			int sortOrder = [[arguments objectForKey:@"sortOrder"] intValue];
#if MAC_OS_X_VERSION_MIN_REQUIRED < MAC_OS_X_VERSION_10_6
			if (sortOrder == ascendingOrder) {
				[orderedEntries sortUsingSelector:@selector(compareDiaryEntry:)];
			} else {
				[orderedEntries sortUsingSelector:@selector(reverseCompareDiaryEntry:)];
			}
#else
			// Blocks-based sort method, added in Mac OS X 10.6 Snow Leopard.
			if (sortOrder == ascendingOrder) {
				[orderedEntries sortUsingComparator:^(id entry1, id entry2) {
					NSTimeInterval interval = [[entry1 date] timeIntervalSinceDate:[entry2 date]];
					if (interval < 0.0) {
						return (NSComparisonResult)NSOrderedAscending;
					} else if (interval > 0.0) {
						return (NSComparisonResult)NSOrderedDescending;
					}
					return (NSComparisonResult)NSOrderedSame;
				}];
			} else {
				[orderedEntries sortUsingComparator:^(id entry2, id entry1) {
					NSTimeInterval interval = [[entry1 date] timeIntervalSinceDate:[entry2 date]];
					if (interval < 0.0) {
						return (NSComparisonResult)NSOrderedAscending;
					} else if (interval > 0.0) {
						return (NSComparisonResult)NSOrderedDescending;
					}
					return (NSComparisonResult)NSOrderedSame;
				}];
			}
#endif
			
			// Build new diary document text storage from sorted entries.
			NSTextStorage *oldStorage = [(DiaryDocument *)document diaryDocTextStorage];
			NSTextStorage *newStorage = [[NSTextStorage alloc] init];
			for (DiaryEntry *thisEntry in orderedEntries) {
				[newStorage appendAttributedString:[thisEntry entryText]];
				if ([orderedEntries indexOfObject:thisEntry] + 1 < [orderedEntries count]) {
					[newStorage appendAttributedString:[[[NSAttributedString alloc] initWithString:@"\n\n"] autorelease]];
				}
			}
			
			// Install and display new diary document text storage with undo/redo support.
			NSRange documentRange = NSMakeRange(0, [oldStorage length]);
			NSString *actionName = NSLocalizedString(@"Sort Diary Entries", @"name of undo action for AppleScript sort diary entries");
			[(DiaryDocument *)document updateDocumentInRange: documentRange withText:newStorage undoActionName:actionName];
			[newStorage release];
		}
	}
	return nil; // no result
}

@end
