//
//  DiaryPrintPanelAccessoryController.m
//  Vermont Recipes 2.0.0
//
//  Created by Bill Cheeseman on 2009-11-03.
//  Copyright 2000-2009 Bill Cheeseman. All rights reserved.
//
//  CREATED IN RECIPE 9

#import "DiaryPrintPanelAccessoryController.h"
#import "DiaryDocument.h" // ADDED IN RECIPE 9
#import "DiaryWindowController.h" // ADDED IN RECIPE 9


@implementation DiaryPrintPanelAccessoryController

#pragma mark INITIALIZATION

- (void)dealloc { // ADDED IN RECIPE 10
	NSNotificationCenter *defaultCenter = [NSNotificationCenter defaultCenter];	
	if (floor(NSFoundationVersionNumber) <= NSFoundationVersionNumber10_5) {
		[defaultCenter removeObserver:self];
	} else {
		[defaultCenter removeObserver:[self userDefaultsDidChangeObserver]];
	}
	[super dealloc];
}

#pragma mark FACTORY METHOD

+ (DiaryPrintPanelAccessoryController *)sharedController { // ADDED IN RECIPE 9
	static DiaryPrintPanelAccessoryController *sharedController = nil;
	if (sharedController == nil) {
		sharedController = [[self alloc] initWithNibName:@"DiaryPrintPanelAccessoryView" bundle:nil];
	}
	return sharedController;
}

#pragma mark ACCESSOR METHODS

- (NSMatrix *)printItemsRadioGroup { // ADDED IN RECIPE 9
	return printItemsRadioGroup;
}

- (NSButton *)printTagsCheckbox { // ADDED IN RECIPE 9
	return printTagsCheckbox;
}

- (NSButton *)printHeadersAndFootersCheckbox { // ADDED IN RECIPE 9
	return printHeadersAndFootersCheckbox;
}

- (NSMatrix *)printTimestampRadioGroup { // ADDED IN RECIPE 9
	return printTimestampRadioGroup;
}

- (void)setPrintItems:(NSInteger)option { // ADDED IN RECIPE 9
	// 0 = Print all entries; 1 = Print current entry; 2 = Print selected text.
	[[[self representedObject] dictionary] setObject:[NSNumber numberWithInteger:option] forKey:VRDefaultDiaryPrintItemsKey];
}

- (NSInteger)printItems { // ADDED IN RECIPE 9
	return [[[[self representedObject] dictionary] objectForKey:VRDefaultDiaryPrintItemsKey] integerValue];
}

- (void)setPrintTags:(BOOL)flag { // ADDED IN RECIPE 9
	[[[self representedObject] dictionary] setObject:[NSNumber numberWithBool:flag] forKey:VRDefaultDiaryPrintTagsKey];
}

- (BOOL)printTags { // ADDED IN RECIPE 9
	return [[[[self representedObject] dictionary] objectForKey:VRDefaultDiaryPrintTagsKey] boolValue];
}

- (void)setPrintHeadersAndFooters:(BOOL)flag { // ADDED IN RECIPE 9
	[[[self representedObject] dictionary] setObject:[NSNumber numberWithBool:flag] forKey:VRDefaultDiaryPrintHeadersAndFootersKey];
}

- (BOOL)printHeadersAndFooters { // ADDED IN RECIPE 9
	return [[[[self representedObject] dictionary] objectForKey:VRDefaultDiaryPrintHeadersAndFootersKey] boolValue];
}

- (void)setPrintTimestamp:(NSInteger)option { // ADDED IN RECIPE 9
	// 0 = Date printed; 1 = Date saved.
	[[[self representedObject] dictionary] setObject:[NSNumber numberWithInteger:option] forKey:VRDefaultDiaryPrintTimestampKey];
}

- (NSInteger)printTimestamp { // ADDED IN RECIPE 9
	return [[[[self representedObject] dictionary] objectForKey:VRDefaultDiaryPrintTimestampKey] integerValue];
}

- (void)setUserDefaultsDidChangeObserver:(id)observer { // ADDED IN RECIPE 10
	userDefaultsDidChangeObserver = observer;
}

- (id)userDefaultsDidChangeObserver { // ADDED IN RECIPE 10
	return userDefaultsDidChangeObserver;
}

#pragma mark ACTION METHODS

- (IBAction)changePrintItems:(id)sender { // ADDED IN RECIPE 9
	[self setPrintItems:[sender selectedRow]];
}

- (IBAction)changePrintTags:(id)sender { // ADDED IN RECIPE 9
	[self setPrintTags:[sender state] ? YES : NO];
}

- (IBAction)changePrintHeadersAndFooters:(id)sender { // ADDED IN RECIPE 9
	[self setPrintHeadersAndFooters:[sender state] ? YES : NO];
}

- (IBAction)changePrintTimestamp:(id)sender { // ADDED IN RECIPE 9
	[self setPrintTimestamp:[sender selectedRow]];
}

#pragma mark OVERRIDE METHODS

- (void)loadView { // ADDED IN RECIPE 9
	[super loadView];
	[self setTitle:NSLocalizedString(@"Vermont Recipes Chef’s Diary", @"title of custom print accessory view for Chef's Diary")];
	// [self updateView];
	// [self performSelector:@selector(updateView) withObject:nil afterDelay:0.0];
	
	// Register notification observers for changes to Printing section of Chef's Diary pane of the application preferences window;	ADDED IN RECIPE 10.
	NSNotificationCenter *defaultCenter = [NSNotificationCenter defaultCenter];
#if MAC_OS_X_VERSION_MIN_REQUIRED < MAC_OS_X_VERSION_10_6
	[defaultCenter addObserver:self selector:@selector(userDefaultsDidChange:) name:NSUserDefaultsDidChangeNotification  object:nil]; // ADDED IN RECIPE 10
#else
	[self setUserDefaultsDidChangeObserver:[defaultCenter addObserverForName:NSUserDefaultsDidChangeNotification object:nil queue:nil usingBlock:^(NSNotification *notification) {
		[self userDefaultsDidChange:notification];
	}]];
#endif	
}

- (void)setRepresentedObject:(id)printInfo { // ADDED IN RECIPE 9
    [super setRepresentedObject:printInfo];
	[self updateView];
}

#pragma mark NOTIFICATION METHODS

- (void)userDefaultsDidChange:(NSNotification *)notification { // ADDED IN RECIPE 10
	// Notification method triggered by NSUserDefaultsDidChangeNotification.
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	[[self printTagsCheckbox] setState:([defaults boolForKey:VRDefaultDiaryPrintTagsKey]) ? NSOnState : NSOffState];
	[[self printHeadersAndFootersCheckbox] setState:([defaults boolForKey:VRDefaultDiaryPrintHeadersAndFootersKey]) ? NSOnState : NSOffState];
	[[self printTimestampRadioGroup] setState:NSOnState atRow:[defaults integerForKey:VRDefaultDiaryPrintTimestampKey] column:0];
}

#pragma mark PROTOCOL METHODS

- (NSSet *)keyPathsForValuesAffectingPreview { // ADDED IN RECIPE 9
	// Key paths to update print preview.
	return [NSSet setWithObjects:@"printItems", @"printTags", @"printHeadersAndFooters", @"printTimestamp", nil];
}

- (NSArray *)localizedSummaryItems { // ADDED IN RECIPE 9
	// Array of localized strings for application-specific settings displayed in the Summary pane of the print panel.
	NSString *titleForPrintItems = NSLocalizedString(@"Items to Print", @"print panel summary title for print items radio group");
	NSString *valueForPrintItems = @"";
	switch ([self printItems]) {
		case 0:
			valueForPrintItems = NSLocalizedString(@"All entries", @"print panel summary value for first radio button in print items radio group");
			break;
		case 1:
			valueForPrintItems = NSLocalizedString(@"Current entry", @"print panel summary value for second radio button in print items radio group");
			break;
		case 2:
			valueForPrintItems = NSLocalizedString(@"Selected text", @"print panel summary value for third radio button in print items radio group");
			break;
	}
	NSString *titleForPrintTags = NSLocalizedString(@"Tag Lists", @"print panel summary title for print tag lists checkbox");
	NSString *valueForPrintTags = [self printTags] ? NSLocalizedString(@"On", @"print panel summary value for boolean setting true") : NSLocalizedString(@"Off", @"print panel summary value for boolean setting false");
	NSString *titleForPrintHeadersAndFooters = NSLocalizedString(@"Headers and Footers", @"print panel summary title for print headers and footers checkbox");
	NSString *valueForPrintHeadersAndFooters = [self printHeadersAndFooters] ? NSLocalizedString(@"On", @"print panel summary value for boolean setting true") : NSLocalizedString(@"Off", @"print panel summary value for boolean setting false");
	NSString *titleForPrintTimestamp = NSLocalizedString(@"Timestamp", @"print panel summary title for print timestamp radio group");
	NSString *valueForPrintTimestamp = @"";
	switch ([self printTimestamp]) {
		case 0:
			valueForPrintTimestamp = NSLocalizedString(@"Date printed", @"print panel summary value for first radio button in timestamp radio group");
			break;
		case 1:
			valueForPrintTimestamp = NSLocalizedString(@"Date saved", @"print panel summary value for second radio button in timestamp radio group");
			break;
	}
    return [NSArray arrayWithObjects:
			[NSDictionary dictionaryWithObjectsAndKeys:titleForPrintItems,NSPrintPanelAccessorySummaryItemNameKey, valueForPrintItems, NSPrintPanelAccessorySummaryItemDescriptionKey, nil],
			[NSDictionary dictionaryWithObjectsAndKeys:titleForPrintTags, NSPrintPanelAccessorySummaryItemNameKey, valueForPrintTags, NSPrintPanelAccessorySummaryItemDescriptionKey, nil], 
			[NSDictionary dictionaryWithObjectsAndKeys:titleForPrintHeadersAndFooters, NSPrintPanelAccessorySummaryItemNameKey, valueForPrintHeadersAndFooters, NSPrintPanelAccessorySummaryItemDescriptionKey, nil],
			[NSDictionary dictionaryWithObjectsAndKeys:titleForPrintTimestamp, NSPrintPanelAccessorySummaryItemNameKey, valueForPrintTimestamp, NSPrintPanelAccessorySummaryItemDescriptionKey, nil], nil];
}

#pragma mark VIEW MANAGEMENT

- (void)updateView { // ADDED IN RECIPE 9
	// NSLog(@"in -updateView; represented object exists: %@", [self representedObject] ? @"YES" : @"NO");
	
	// set user interface element states to match values in represented object (print info).
	[[self printItemsRadioGroup] setState:NSOnState atRow:[self printItems] column:0];
	[[self printTagsCheckbox] setState:([self printTags]) ? NSOnState : NSOffState];
	[[self printHeadersAndFootersCheckbox] setState:([self printHeadersAndFooters]) ? NSOnState : NSOffState];
	[[self printTimestampRadioGroup] setState:NSOnState atRow:[self printTimestamp] column:0];

	// Disable radio buttons based on diary document selection or insertion point.
	DiaryDocument *document = [[NSDocumentController sharedDocumentController] currentDocument];
	DiaryWindowController *windowController = [[document windowControllers] objectAtIndex:0];
	NSUInteger insertionPointIndex = [windowController insertionPointIndex];
	BOOL isCurrentEntry = [document currentEntryTitleRangeForIndex:insertionPointIndex].location != NSNotFound;
	[[[self printItemsRadioGroup] cellAtRow:1 column:0] setEnabled:isCurrentEntry];
	NSArray *selection = [[windowController keyDiaryView] selectedRanges];
	BOOL isSelection = (([selection count] > 1) || ([[selection objectAtIndex:0] rangeValue].length > 0));
	[[[self printItemsRadioGroup] cellAtRow:2 column:0] setEnabled:isSelection];	
}

@end

NSString *VRDefaultDiaryPrintItemsKey = @"diary print items"; // ADDED IN RECIPE 9
NSString *VRDefaultDiaryPrintTagsKey = @"diary print tags"; // ADDED IN RECIPE 9
NSString *VRDefaultDiaryPrintHeadersAndFootersKey = @"diary print headers and footers"; // ADDED IN RECIPE 9
NSString *VRDefaultDiaryPrintTimestampKey = @"diary print timestamp"; // ADDED IN RECIPE 9
