//
//  DiaryWindowController.h
//  Vermont Recipes 2.0.0

//  Created by Bill Cheeseman on 2009-06-05.
//  Copyright 2000-2009 Bill Cheeseman. All rights reserved.
//
//  CREATED IN RECIPE 3

#import <Cocoa/Cocoa.h>
#import "ValidationProtocols.h" // ADDED IN RECIPE 10


extern NSString *VRDefaultDiaryWindowStandardSizeKey; // ADDED IN RECIPE 7
extern NSString *VRDiaryWindowAutosaveName; // ADDED IN RECIPE 7
extern NSString *VRDiarySplitViewAutosaveName; // ADDED IN RECIPE 7

extern NSString *VRDefaultSuppressRestoreDiaryDocumentAlertKey; // ADDED IN RECIPE 10

@interface DiaryWindowController : NSWindowController <NSUserInterfaceValidations, NSMenuDelegate> { // EDITED IN RECIPE 4 and RECIPE 8
	IBOutlet NSTextView *diaryView; // ADDED IN RECIPE 3
	IBOutlet NSTextView *otherDiaryView; // ADDED IN RECIPE 3
	IBOutlet NSDatePicker *datePicker; // ADDED IN RECIPE 4
	IBOutlet NSSearchField *searchField; // ADDED IN RECIPE 5
	IBOutlet NSSplitView *splitView; // ADDED IN RECIPE 7
	IBOutlet NSButton *addTagButton; //	ADDED IN RECIPE 8
	
	id eventMonitor; // ADDED IN RECIPE 8
	
	id didSaveDiaryDocumentObserver; // ADDED IN RECIPE 8
	id didAutosaveDiaryDocumentObserver; // ADDED IN RECIPE 8
	id didRestoreAutosavedDiaryDocumentObserver; // ADDED IN RECIPE 8
	
	NSAlert *restoreAutosavedDiaryDocumentAlert; // ADDED IN RECIPE 10
	id userDefaultsDidChangeObserver; // ADDED IN RECIPE 10
}

/* REMOVED IN RECIPE 14:
#pragma mark ACCESSOR METHODS

- (NSTextView *)diaryView; // ADDED IN RECIPE 3
- (NSTextView *)otherDiaryView; // ADDED IN RECIPE 3
- (NSDatePicker *)datePicker; // ADDED IN RECIPE 4
- (NSSearchField *)searchField; // ADDED IN RECIPE 5
- (NSSplitView *)splitView; // ADDED IN RECIPE 7
- (NSButton *)addTagButton; // ADDED IN RECIPE 8

- (void)setEventMonitor:(id)eventMonitor; // ADDED IN RECIPE 8
- (id)eventMonitor; // ADDED IN RECIPE 8

- (void)setDidSaveDiaryDocumentObserver:(id)observer; // ADDED IN RECIPE 8
- (id)didSaveDiaryDocumentObserver; // ADDED IN RECIPE 8

- (void)setDidAutosaveDiaryDocumentObserver:(id)observer; // ADDED IN RECIPE 8
- (id)didAutosaveDiaryDocumentObserver; // ADDED IN RECIPE 8

- (void)setDidRestoreAutosavedDiaryDocumentObserver:(id)observer; // ADDED IN RECIPE 8
- (id)didRestoreAutosavedDiaryDocumentObserver; // ADDED IN RECIPE 8

- (void)setRestoreAutosavedDiaryDocumentAlert:(NSAlert *)alert; // ADDED IN RECIPE 10
- (NSAlert *)restoreAutosavedDiaryDocumentAlert; // ADDED IN RECIPE 10

- (void)setUserDefaultsDidChangeObserver:(id)observer; // ADDED IN RECIPE 10
- (id)userDefaultsDidChangeObserver; // ADDED IN RECIPE 10
*/

#pragma mark PROPERTIES

@property (readonly, nonatomic) IBOutlet NSTextView *diaryView; // ADDED IN RECIPE 14
@property (readonly, nonatomic) IBOutlet NSTextView *otherDiaryView; // ADDED IN RECIPE 14
@property (readonly, nonatomic) IBOutlet NSDatePicker *datePicker; // ADDED IN RECIPE 14
@property (readonly, nonatomic) IBOutlet NSSearchField *searchField; // ADDED IN RECIPE 14
@property (readonly, nonatomic) IBOutlet NSSplitView *splitView; // ADDED IN RECIPE 14
@property (readonly, nonatomic) IBOutlet NSButton *addTagButton; // ADDED IN RECIPE 14
@property (assign, nonatomic) id eventMonitor; // ADDED IN RECIPE 14
@property (assign, nonatomic) id didSaveDiaryDocumentObserver; // ADDED IN RECIPE 14
@property (assign, nonatomic) id didAutosaveDiaryDocumentObserver; // ADDED IN RECIPE 14
@property (assign, nonatomic) id didRestoreAutosavedDiaryDocumentObserver; // ADDED IN RECIPE 14
@property (assign, nonatomic) id restoreAutosavedDiaryDocumentAlert; // ADDED IN RECIPE 14
@property (assign, nonatomic) id userDefaultsDidChangeObserver; // ADDED IN RECIPE 14

#pragma mark ACTION METHODS

- (IBAction)addEntry:(id)sender; // ADDED IN RECIPE 4
- (IBAction)addTag:(id)sender; // ADDED IN RECIPE 4
- (IBAction)tagAll:(id)sender; // ADDED IN RECIPE 8

- (IBAction)goToFirstEntry:(id)sender; // ADDED IN RECIPE 4
- (IBAction)goToLastEntry:(id)sneder; // ADDED IN RECIPE 4
- (IBAction)goToPreviousEntry:(id)sender; // ADDED IN RECIPE 4
- (IBAction)goToNextEntry:(id)sender; // ADDED IN RECIPE 4
// - (IBAction)goToDatedEntry:(id)sender; // ADDED IN RECIPE 4; REMOVED IN RECIPE 14
- (IBAction)findTag:(id)sender; // ADDED IN RECIPE 4

#pragma mark NOTIFICATION METHODS

- (void)didSaveDiaryDocument:(NSNotification *)notification; // ADDED IN RECIPE 8
- (void)didAutosaveDiaryDocument:(NSNotification *)notification; // ADDED IN RECIPE 8
- (void)didRestoreAutosavedDiaryDocument:(NSNotification *)notification; // ADDED IN RECIPE 8

#pragma mark USER INTERFACE VALIDATION

- (void)updateWindow; // ADDED IN RECIPE 4
// - (void)updateDatePickerValue; // ADDED IN RECIPE 4; REMOVED IN RECIPE 14

#pragma mark UTILITY METHODS

- (NSTextView *)keyDiaryView; // ADDED IN RECIPE 4
- (NSUInteger)insertionPointIndex; // ADDED IN RECIPE 4
- (void)toggleAddTagButtonForModifierFlags:(NSUInteger)modifierFlags; // ADDED IN RECIPE 8

#pragma mark ALERTS

- (NSAlert *)alertDidRestoreAutosavedDiaryDocument; // ADDED IN RECIPE 7
- (void)userDefaultsDidChange:(NSNotification *)notification; // ADDED IN RECIPE 10

@end


/* MOVED TO ValidationProtocols.h IN RECIPE 10
#pragma mark -

@protocol VRValidatedControl <NSValidatedUserInterfaceItem> // ADDED IN RECIPE 4
@required
- (void)validate;
@end


#pragma mark -

@protocol VRControlValidations // ADDED IN RECIPE 4
@optional
- (BOOL)validateControl:(id <VRValidatedControl>)item;
@end
*/

#pragma mark -

@interface ValidatedDiaryButton : NSButton <VRValidatedControl> { // ADDED IN RECIPE 4
}
@end


/* REMOVED IN RECIPE 14
#pragma mark -

@interface ValidatedDiaryDatePicker : NSDatePicker <VRValidatedControl> { // ADDED IN RECIPE 4
}
@end
*/

#pragma mark -

@interface ValidatedDiarySearchField : NSSearchField <VRValidatedControl> { // ADDED IN RECIPE 4
}
@end
